* src/autofit/*: more updates
1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 86 87 88 89 90 91 92 93 94 95 96 97 98 99 100 101 102 103 104 105 106 107 108 109 110 111 112 113 114 115 116 117 118 119 120 121 122 123 124 125 126 127 128 129 130 131 132 133 134 135 136 137 138 139 140 141 142 143 144 145 146 147 148 149 150 151 152 153 154 155 156 157 158 159 160 161 162 163 164 165 166 167 168 169 170 171 172 173 174 175 176 177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 192 193 194 195 196 197 198 199 200 201 202 203 204 205 206 207 208 209 210 211 212 213 214 215 216 217 218 219 220 221 222 223 224 225 226 227 228 229 230 231 232 233 234 235 236 237 238 239 240 241 242 243 244 245 246 247 248 249 250 251 252 253 254 255 256 257 258 259 260 261 262 263 264 265 266 267 268 269 270 271 272 273 274 275 276 277 278 279 280 281 282 283 284 285 286 287 288 289 290 291 292 293 294 295 296 297 298 299 300 301 302 303 304 305 306 307 308 309 310 311 312 313 314 315 316 317 318 319 320 321 322 323 324 325 326 327 328 329 330 331 332 333 334 335 336 337 338 339 340 341 342 343 344 345 346 347 348 349 350 351 352 353 354 355 356 357 358 359 360 361 362 363 364 365 366 367 368 369 370 371 372 373 374 375 376 377 378 379 380 381 382 383 384 385 386 387 388 389 390 391 392 393 394 395 396 397 398 399 400 401 402 403 404 405 406 407 408 409 410 411 412 413 414 415 416 417 418 419 420 421 422 423 424 425 426 427 428 429 430 431 432 433 434 435 436 437 438 439 440 441 442 443 444 445 446 447 448 449 450 451 452 453 454 455 456 457 458 459 460 461 462 463 464 465 466 467 468 469 470 471 472 473 474 475 476 477 478 479 480 481 482 483 484 485 486 487 488 489 490 491 492 493 494 495 496 497 498 499 500 501 502 503 504 505 506 507 508 509 510 511 512 513 514 515 516 517 518 519 520 521 522 523 524 525 526 527 528 529 530 531 532 533 534 535 536 537 538 539 540 541 542 543 544 545 546 547 548 549 550 551 552 553 554 555 556 557 558 559 560 561 562 563 564 565 566 567 568 569 570 571 572 573 574 575 576 577 578 579 580 581 582 583 584 585 586 587 588 589 590 591 592 593 594 595 596 597 598 599 600 601 602 603 604 605 606 607 608 609 610 611 612 613 614 615 616 617 618 619 620 621 622 623 624 625 626 627 628 629 630 631 632 633 634 635 636 637 638 639 640 641 642 643 644 645 646 647 648 649 650 651 652 653 654 655 656 657 658 659 660 661 662 663 664 665 666 667 668 669 670 671 672 673 674 675 676 677 678 679 680 681 682 683 684 685 686 687 688 689 690 691 692 693 694 695 696 697 698 699 700 701 702 703 704 705 706 707 708 709 710 711 712 713 714 715 716 717 718 719 720 721 722 723 724 725 726 727 728 729 730 731 732 733 734 735 736 737 738 739 740 741 742 743 744 745 746 747 748 749 750 751 752 753 754 755 756 757 758 759 760 761 762 763 764 765 766 767 768 769 770 771 772 773 774 775 776 777 778 779 780 781 782 783 784 785 786 787 788 789 790 791 792 793 794 795 796 797 798 799 800 801 802 803 804 805 806 807 808 809 810 811 812 813 814 815 816 817 818 819 820 821 822 823 824 825 826 827 828 829 830 831 832 833 834 835 836 837 838 839 840 841 842 843 844 845 846 847 848 849 850 851 852 853 854 855 856 857 858 859 860 861 862 863 864 865 866 867 868 869 870 871 872 873 874 875 876 877 878 879 880 881 882 883 884 885 886 887 888 889 890 891 892 893 894 895 896 897 898 899 900 901 902 903 904 905 906 907 908 909 910 911 912 913 914 915 916 917 918 919 920 921 922 923 924 925 926 927 928 929 930 931 932 933 934 935 936 937 938 939 940 941 942 943 944 945 946 947 948 949 950 951 952 953 954 955 956 957 958 959 960 961 962 963 964 965 966 967 968 969 970 971 972 973 974 975 976 977 978 979 980 981 982 983 984 985 986 987 988 989 990 991 992 993 994 995 996 997 998 999 1000 1001 1002 1003 1004 1005 1006 1007 1008 1009 1010 1011 1012 1013 1014 1015 1016 1017 1018 1019 1020 1021 1022 1023 1024 1025 1026 1027 1028 1029 1030 1031 1032 1033 1034 1035 1036 1037 1038 1039 1040 1041 1042 1043 1044 1045 1046 1047 1048 1049 1050 1051 1052 1053 1054 1055 1056 1057 1058 1059 1060 1061 1062 1063 1064 1065 1066 1067 1068 1069 1070 1071 1072 1073 1074 1075 1076 1077 1078 1079 1080 1081 1082 1083 1084 1085 1086 1087 1088 1089 1090 1091 1092 1093 1094 1095 1096 1097 1098 1099 1100 1101 1102 1103 1104 1105 1106 1107 1108 1109 1110 1111 1112 1113 1114 1115 1116 1117 1118 1119 1120 1121 1122 1123 1124 1125 1126 1127 1128 1129 1130 1131 1132 1133 1134 1135 1136 1137 1138 1139 1140 1141 1142 1143 1144 1145 1146 1147 1148 1149 1150 1151 1152 1153 1154 1155 1156 1157 1158 1159 1160 1161 1162 1163 1164 1165 1166 1167 1168 1169 1170 1171 1172 1173 1174 1175 1176 1177 1178 1179 1180 1181 1182 1183 1184 1185 1186 1187 1188 1189 1190 1191 1192 1193 1194 1195 1196 1197 1198 1199 1200 1201 1202 1203 1204 1205 1206 1207 1208 1209 1210 1211 1212 1213 1214 1215 1216 1217 1218 1219 1220 1221 1222 1223 1224 1225 1226 1227 1228 1229 1230 1231 1232 1233 1234 1235 1236 1237 1238 1239 1240 1241 1242 1243 1244 1245 1246 1247 1248 1249 1250 1251 1252 1253 1254 1255 1256 1257 1258 1259 1260 1261 1262 1263 1264 1265 1266 1267 1268 1269 1270 1271 1272 1273 1274 1275 1276 1277 1278 1279 1280 1281 1282 1283 1284 1285 1286 1287 1288 1289 1290 1291 1292 1293 1294 1295 1296 1297 1298 1299 1300 1301 1302 1303 1304 1305 1306 1307 1308 1309 1310 1311 1312 1313 1314 1315 1316 1317 1318 1319 1320 1321 1322 1323 1324 1325 1326 1327 1328 1329 1330 1331 1332 1333 1334 1335 1336 1337 1338 1339 1340 1341 1342 1343 1344 1345 1346 1347 1348 1349 1350 1351 1352 1353 1354 1355 1356 1357 1358 1359 1360 1361 1362 1363 1364 1365 1366 1367 1368 1369 1370 1371 1372 1373 1374 1375 1376 1377 1378 1379 1380 1381 1382 1383 1384 1385 1386 1387 1388 1389 1390 1391 1392 1393 1394 1395 1396 1397 1398 1399 1400 1401 1402 1403 1404 1405 1406 1407 1408 1409 1410 1411 1412 1413 1414 1415 1416 1417 1418 1419 1420 1421 1422 1423 1424 1425 1426 1427 1428 1429 1430 1431 1432 1433 1434 1435 1436 1437 1438 1439 1440 1441 1442 1443 1444 1445 1446 1447 1448 1449 1450 1451 1452 1453 1454 1455 1456 1457 1458 1459 1460 1461 1462 1463 1464 1465 1466 1467 1468 1469 1470 1471 1472 1473 1474 1475 1476 1477 1478 1479 1480 1481 1482 1483 1484 1485 1486 1487 1488 1489 1490 1491 1492 1493 1494 1495 1496 1497 1498 1499 1500 1501 1502 1503 1504 1505 1506 1507 1508 1509 1510 1511 1512 1513 1514 1515 1516 1517 1518 1519 1520 1521 1522 1523 1524 1525 1526 1527 1528 1529 1530 1531 1532 1533 1534 1535 1536 1537 1538 1539 1540 1541 1542 1543 1544 1545 1546 1547 1548 1549 1550 1551 1552 1553 1554 1555 1556 1557 1558 1559 1560 1561 1562 1563 1564 1565 1566 1567 1568 1569 1570 1571 1572 1573 1574 1575 1576 1577 1578 1579 1580 1581 1582 1583 1584 1585 1586 1587 1588 1589 1590 1591 1592 1593 1594 1595 1596 1597 1598 1599 1600 1601 1602 1603 1604 1605 1606 1607 1608 1609 1610 1611 1612 1613 1614 1615 1616 1617 1618 1619 1620 1621 1622 1623 1624 1625 1626 1627 1628 1629 1630 1631 1632 1633 1634 1635 1636 1637 1638 1639 1640 1641 1642 1643 1644 1645 1646 1647 1648 1649 1650 1651 1652 1653 1654 1655 1656 1657 1658 1659 1660 1661 1662 1663 1664 1665 1666 1667 1668 1669 1670 1671 1672 1673 1674 1675 1676 1677 1678 1679 1680 1681 1682 1683 1684 1685 1686 1687 1688 1689 1690 1691 1692 1693 1694 1695 1696 1697 1698 1699 1700 1701 1702 1703 1704 1705 1706 1707 1708 1709 1710 1711 1712 1713 1714 1715 1716 1717 1718 1719 1720 1721 1722 1723 1724 1725 1726 1727 1728 1729 1730 1731 1732 1733 1734 1735 1736 1737 1738 1739 1740 1741 1742 1743 1744 1745 1746 1747 1748 1749 1750 1751 1752 1753 1754 1755 1756 1757 1758 1759 1760 1761 1762 1763 1764 1765 1766 1767 1768 1769 1770 1771 1772 1773 1774 1775 1776 1777 1778 1779 1780 1781 1782 1783 1784 1785 1786 1787 1788 1789 1790 1791 1792 1793 1794 1795 1796 1797 1798 1799 1800 1801 1802 1803 1804 1805 1806 1807 1808 1809 1810 1811 1812 1813 1814 1815 1816 1817 1818 1819 1820 1821 1822 1823 1824 1825 1826 1827 1828 1829 1830 1831 1832 1833 1834 1835 1836 1837 1838 1839 1840 1841 1842 1843 1844 1845 1846 1847 1848 1849 1850 1851 1852 1853 1854 1855 1856 1857 1858 1859 1860 1861 1862 1863 1864 1865 1866 1867 1868 1869 1870 1871 1872 1873 1874 1875 1876 1877 1878 1879 1880 1881 1882 1883 1884 1885 1886 1887 1888 1889 1890 1891 1892 1893 1894 1895 1896 1897 1898 1899 1900 1901 1902 1903 1904 1905 1906 1907 1908 1909 1910 1911 1912 1913 1914 1915 1916 1917 1918 1919 1920 1921 1922 1923 1924 1925 1926 1927 1928 1929 1930 1931 1932 1933 1934 1935 1936 1937 1938 1939 1940 1941 1942 1943 1944 1945 1946 1947 1948 1949 1950 1951 1952 1953 1954 1955 1956 1957 1958 1959 1960 1961 1962 1963 1964 1965 1966 1967 1968 1969 1970 1971 1972 1973 1974 1975 1976 1977 1978 1979 1980 1981 1982 1983 1984 1985 1986 1987 1988 1989 1990 1991 1992 1993 1994 1995 1996 1997 1998 1999 2000 2001 2002 2003 2004 2005 2006 2007 2008 2009 2010 2011 2012 2013 2014 2015 2016 2017 2018 2019 2020 2021 2022 2023 2024 2025 2026 2027 2028 2029 2030 2031 2032 2033 2034 2035 2036 2037 2038 2039 2040 2041 2042 2043 2044 2045 2046 2047 2048 2049 2050 2051 2052 2053 2054 2055 2056 2057 2058 2059 2060 2061 2062 2063 2064 2065 2066 2067 2068 2069 2070 2071 2072 2073 2074 2075 2076 2077 2078 2079 2080 2081 2082 2083 2084 2085 2086 2087 2088 2089 2090 2091 2092 2093 2094 2095 2096 2097 2098 2099 2100 2101 2102 2103 2104 2105 2106 2107 2108 2109 2110 2111 2112 2113 2114 2115 2116 2117 2118 2119 2120 2121 2122 2123 2124 2125 2126 2127 2128 2129 2130 2131 2132 2133 2134 2135 2136 2137 2138 2139 2140 2141 2142 2143 2144 2145 2146 2147 2148 2149 2150 2151 2152 2153 2154 2155 2156 2157 2158 2159 2160 2161 2162 2163 2164 2165 2166 2167 2168 2169 2170 2171 2172 2173 2174 2175 2176 2177 2178 2179 2180 2181 2182 2183 2184 2185 2186 2187 2188 2189 2190 2191 2192 2193 2194 2195 2196 2197 2198 2199 2200 2201 2202 2203 2204 2205 2206 2207 2208 2209 2210 2211 2212 2213 2214 2215 2216 2217 2218 2219 2220 2221 2222 2223 2224 2225 2226 2227 2228 2229 2230 2231 2232 2233 2234 2235 2236 2237 2238 2239 2240 2241 2242 2243 2244 2245 2246 2247 2248 2249 2250 2251 2252 2253 2254 2255 2256 2257 2258 2259 2260 2261 2262 2263 2264 2265 2266 2267 2268 2269 2270 2271 2272 2273 2274 2275 2276 2277 2278 2279 2280 2281 2282 2283 2284 2285 2286 2287 2288 2289 2290 2291 2292 2293 2294 2295 2296 2297 2298 2299 2300 2301 2302 2303 2304 2305 2306 2307 2308 2309 2310 2311 2312 2313 2314 2315 2316 2317 2318 2319 2320 2321 2322 2323 2324 2325 2326 2327 2328 2329 2330 2331 2332 2333 2334 2335 2336 2337 2338 2339 2340 2341 2342 2343 2344 2345 2346 2347 2348 2349 2350 2351 2352 2353 2354 2355 2356 2357 2358 2359 2360 2361 2362 2363 2364 2365 2366 2367 2368 2369 2370 2371 2372 2373 2374 2375 2376 2377 2378 2379 2380 2381 2382 2383 2384 2385 2386 2387 2388 2389 2390 2391 2392 2393 2394 2395 2396 2397 2398 2399 2400 2401 2402 2403 2404 2405 2406 2407 2408 2409 2410 2411 2412 2413 2414 2415 2416 2417 2418 2419 2420 2421 2422 2423 2424 2425 2426 2427 2428 2429 2430 2431 2432 2433 2434 2435 2436 2437 2438 2439 2440 2441 2442 2443 2444 2445 2446 2447 2448 2449 2450 2451 2452 2453 2454 2455 2456 2457 2458 2459 2460 2461 2462 2463 2464 2465 2466 2467 2468 2469 2470 2471 2472 2473 2474 2475 2476 2477 2478 2479 2480 2481 2482 2483 2484 2485 2486 2487 2488 2489 2490 2491 2492 2493 2494 2495 2496 2497 2498 2499 2500 2501 2502 2503 2504 2505 2506 2507 2508 2509 2510 2511 2512 2513 2514 2515 2516 2517 2518 2519 2520 2521 2522 2523 2524 2525 2526 2527 2528 2529 2530 2531 2532 2533 2534 2535 2536 2537 2538 2539 2540 2541 2542 2543 2544 2545 2546 2547 2548 2549 2550 2551 2552 2553 2554 2555 2556 2557 2558 2559 2560 2561 2562 2563 2564 2565 2566 2567 2568 2569 2570 2571 2572 2573 2574 2575 2576 2577 2578 2579 2580 2581 2582 2583 2584 2585 2586 2587 2588 2589 2590 2591 2592 2593 2594 2595 2596 2597 2598 2599 2600 2601 2602 2603 2604 2605 2606 2607 2608 2609 2610 2611 2612 2613 2614 2615 2616 2617 2618 2619 2620 2621 2622 2623 2624 2625 2626 2627 2628 2629 2630 2631 2632 2633 2634 2635 2636 2637 2638 2639 2640 2641 2642 2643 2644 2645 2646 2647 2648 2649 2650 2651 2652 2653 2654 2655 2656 2657 2658 2659 2660 2661 2662 2663 2664 2665 2666 2667 2668 2669 2670 2671 2672 2673 2674 2675 2676 2677 2678 2679 2680 2681 2682 2683 2684 2685 2686 2687 2688 2689 2690 2691 2692 2693 2694 2695 2696 2697 2698 2699 2700 2701 2702 2703 2704 2705 2706 2707 2708 2709 2710 2711 2712 2713 2714 2715 2716 2717 2718 2719 2720 2721 2722 2723 2724 2725 2726 2727 2728 2729 2730 2731 2732 2733 2734 2735 2736 2737 2738 2739 2740 2741 2742 2743 2744 2745 2746 2747 2748 2749 2750 2751 2752 2753 2754 2755 2756 2757 2758 2759 2760 2761 2762 2763 2764 2765 2766 2767 2768 2769 2770 2771 2772 2773 2774 2775 2776 2777 2778 2779 2780 2781 2782 2783 2784 2785 2786 2787 2788 2789 2790 2791 2792 2793 2794 2795 2796 2797 2798 2799 2800 2801 2802 2803 2804 2805 2806 2807 2808 2809 2810 2811 2812 2813 2814 2815 2816 2817 2818 2819 2820 2821 2822 2823 2824 2825 2826 2827 2828 2829 2830 2831 2832 2833 2834 2835 2836 2837 2838 2839 2840 2841 2842 2843 2844 2845 2846 2847 2848 2849 2850 2851 2852 2853 2854 2855 2856 2857 2858 2859 2860 2861 2862 2863 2864 2865 2866 2867 2868 2869 2870 2871 2872 2873 2874 2875 2876 2877 2878 2879 2880 2881 2882 2883 2884 2885 2886 2887 2888 2889 2890 2891 2892 2893 2894 2895 2896 2897 2898 2899 2900 2901 2902 2903 2904 2905 2906 2907 2908 2909 2910 2911 2912 2913 2914 2915 2916 2917 2918 2919 2920 2921 2922 2923 2924 2925 2926 2927 2928 2929 2930 2931 2932 2933 2934 2935 2936 2937 2938 2939 2940 2941 2942 2943 2944 2945 2946 2947 2948 2949 2950 2951 2952 2953 2954 2955 2956 2957 2958 2959 2960 2961 2962 2963 2964 2965 2966 2967 2968 2969 2970 2971 2972 2973 2974 2975 2976 2977 2978 2979 2980 2981 2982 2983 2984 2985 2986 2987 2988 2989 2990 2991 2992 2993 2994 2995 2996 2997 2998 2999 3000 3001 3002 3003 3004 3005 3006 3007 3008 3009 3010 3011 3012 3013 3014 3015 3016 3017 3018 3019 3020 3021 3022 3023 3024 3025 3026
diff --git a/ChangeLog b/ChangeLog
index 0269948..84fbdea 100644
--- a/ChangeLog
+++ b/ChangeLog
@@ -1,3 +1,7 @@
+2003-11-22 David Turner <david@freetype.org>
+
+ * src/autofit/*: more updates
+
2003-11-13 John A. Boyd Jr. <jaboydjr@netwalk.com>
* src/bdf/bdfdrivr.c (bdf_interpret_style), src/pcf/pcfread.c
diff --git a/README b/README
index bf99cfd..72baf3d 100644
--- a/README
+++ b/README
@@ -9,7 +9,7 @@
is called `libttf'. They are *not* compatible!
- FreeType 2.1.7
+ FreeType 2.1.8
==============
Please read the docs/CHANGES file, it contains IMPORTANT INFORMATION.
@@ -19,9 +19,9 @@
Note that the FreeType 2 documentation is now available as a
separate package from our sites. See:
- ftp://ftp.freetype.org/pub/freetype2/ftdocs-2.1.7.tar.bz2
- ftp://ftp.freetype.org/pub/freetype2/ftdocs-2.1.7.tar.gz
- ftp://ftp.freetype.org/pub/freetype2/ftdoc216.zip
+ ftp://ftp.freetype.org/pub/freetype2/ftdocs-2.1.8.tar.bz2
+ ftp://ftp.freetype.org/pub/freetype2/ftdocs-2.1.8.tar.gz
+ ftp://ftp.freetype.org/pub/freetype2/ftdoc218.zip
Bugs
diff --git a/builds/unix/configure.ac b/builds/unix/configure.ac
index 90d21f6..89ba574 100644
--- a/builds/unix/configure.ac
+++ b/builds/unix/configure.ac
@@ -8,7 +8,7 @@ AC_CONFIG_SRCDIR([ftconfig.in])
# Don't forget to update docs/VERSION.DLL!
-version_info='9:5:3'
+version_info='9:6:3'
AC_SUBST([version_info])
ft_version=`echo $version_info | tr : .`
AC_SUBST([ft_version])
diff --git a/docs/VERSION.DLL b/docs/VERSION.DLL
index 551fced..9bdd643 100644
--- a/docs/VERSION.DLL
+++ b/docs/VERSION.DLL
@@ -52,6 +52,7 @@ systems, but not all of them:
release libtool so
-------------------------------
+ 2.1.8 9.6.3 6.3.6
2.1.7 9.5.3 6.3.5
2.1.6 9.5.3 6.3.5
2.1.5 9.4.3 6.3.4
diff --git a/include/freetype/freetype.h b/include/freetype/freetype.h
index 285d8ef..0c6e67e 100644
--- a/include/freetype/freetype.h
+++ b/include/freetype/freetype.h
@@ -44,7 +44,7 @@
/* */
#define FREETYPE_MAJOR 2
#define FREETYPE_MINOR 1
-#define FREETYPE_PATCH 7
+#define FREETYPE_PATCH 8
#include <ft2build.h>
diff --git a/src/autofit/afangles.c b/src/autofit/afangles.c
index 66c7d79..eb0a042 100644
--- a/src/autofit/afangles.c
+++ b/src/autofit/afangles.c
@@ -9,6 +9,7 @@
90, 64, 38, 20, 10, 5, 3, 1, 1
};
+
static FT_Int
af_angle_prenorm( FT_Vector* vec )
{
@@ -162,3 +163,29 @@
return delta;
}
+
+ /* well, this needs to be somewhere, right :-)
+ */
+
+ FT_LOCAL_DEF( void )
+ af_sort_pos( FT_UInt count,
+ FT_Pos* table )
+ {
+ FT_Int i, j;
+ FT_Pos swap;
+
+
+ for ( i = 1; i < count; i++ )
+ {
+ for ( j = i; j > 0; j-- )
+ {
+ if ( table[j] > table[j - 1] )
+ break;
+
+ swap = table[j];
+ table[j] = table[j - 1];
+ table[j - 1] = swap;
+ }
+ }
+ }
+
\ No newline at end of file
diff --git a/src/autofit/afglobal.c b/src/autofit/afglobal.c
new file mode 100644
index 0000000..6f1540c
--- /dev/null
+++ b/src/autofit/afglobal.c
@@ -0,0 +1,231 @@
+#include "afglobal.h"
+#include "aflatin.h"
+
+ /* populate this list when you add new scripts
+ */
+ static AF_ScriptClass const af_script_class_list[] =
+ {
+ & af_latin_script_class,
+
+ NULL /* do not remove */
+ };
+
+#define AF_SCRIPT_LIST_DEFAULT 0 /* index of default script in 'af_script_class_list' */
+#define AF_SCRIPT_LIST_NONE 255 /* indicates an uncovered glyph */
+
+ /*
+ * note that glyph_scripts[] is used to map each glyph into
+ * an index into the 'af_script_class_list' array.
+ *
+ */
+ typedef struct AF_FaceGlobalsRec_
+ {
+ FT_Face face;
+ FT_UInt glyph_count; /* same as face->num_glyphs */
+ FT_Byte* glyph_scripts;
+
+ AF_ScriptMetrics metrics[ AF_SCRIPT_MAX ];
+
+ } AF_FaceGlobalsRec, *AF_FaceGlobals;
+
+
+
+
+ /* this function is used to compute the script index of each glyph
+ * within a given face
+ */
+ static FT_Error
+ af_face_globals_compute_script_coverage( AF_FaceGlobals globals )
+ {
+ FT_Error error = 0;
+ FT_Face face = globals->face;
+ FT_CharMap old_charmap = face->charmap;
+ FT_Byte* gscripts = globals->glyph_scripts;
+ FT_UInt ss;
+
+ /* the value 255 means "uncovered glyph"
+ */
+ FT_MEM_SET( globals->glyph_scripts,
+ AF_SCRIPT_LIST_NONE,
+ globals->glyph_count );
+
+ error = FT_Select_Charmap( face, FT_ENCODING_UNICODE );
+ if ( error )
+ {
+ /* ignore this error, we'll simply use Latin as the standard
+ * script. XXX: Shouldn't we rather disable hinting ??
+ */
+ error = 0;
+ goto Exit;
+ }
+
+ /* scan each script in a Unicode charmap
+ */
+ for ( ss = 0; af_script_classes[ss]; ss++ )
+ {
+ AF_ScriptClass clazz = af_script_classes[ss];
+ AF_Script_UniRange range;
+
+ /* scan all unicode points in the range, and set the corresponding
+ * glyph script index
+ */
+ for ( range = clazz->script_uni_ranges; range->first != 0; range++ )
+ {
+ FT_ULong charcode = range->first;
+ FT_UInt gindex;
+
+ gindex = FT_Get_Char_Index( face, charcode );
+
+ if ( gindex != 0 &&
+ gindex < globals->glyph_count &&
+ gscripts[ gindex ] == AF_SCRIPT_LIST_NONE )
+ {
+ gscripts[ gindex ] = (FT_Byte) ss;
+ }
+ for (;;)
+ {
+ charcode = FT_Get_Next_Char( face, charcode, &gindex );
+
+ if ( gindex == 0 || charcode > range->last )
+ break;
+
+ if ( gindex < globals->glyph_count &&
+ gscripts[ gindex ] == AF_SCRIPT_LIST_NONE )
+ {
+ gscripts[ gindex ] = (FT_Byte) ss;
+ }
+ }
+ }
+ }
+
+ Exit:
+ /* by default, all uncovered glyphs are set to the latin script
+ * XXX: shouldnt' we disable hinting or do something similar ?
+ */
+ {
+ FT_UInt nn;
+
+ for ( nn = 0; nn < globals->glyph_count; nn++ )
+ {
+ if ( gscripts[ nn ] == AF_SCRIPT_LIST_NONE )
+ gscripts[ nn ] = AF_SCRIPT_LIST_DEFAULT;
+ }
+ }
+
+ FT_Set_Charmap( face, old_charmap );
+ return error;
+ }
+
+
+
+ FT_LOCAL_DEF( FT_Error )
+ af_face_globals_new( FT_Face face,
+ AF_FaceGlobals *aglobals )
+ {
+ FT_Error error;
+ FT_Memory memory;
+ AF_FaceGlobals globals;
+
+ memory = face->memory;
+
+ if ( !FT_ALLOC( globals, sizeof(*globals) +
+ face->num_glyphs*sizeof(FT_Byte) ) )
+ {
+ globals->face = face;
+ globals->glyph_count = face->num_glyphs;
+ globals->glyph_scripts = (FT_Byte*)( globals+1 );
+
+ error = af_face_globals_compute_script_coverage( globals );
+ if ( error )
+ {
+ af_face_globals_free( globals );
+ globals = NULL;
+ }
+ }
+
+ Exit:
+ *aglobals = globals;
+ return error;
+ }
+
+
+ FT_LOCAL_DEF( void )
+ af_face_globals_free( AF_FaceGlobals globals )
+ {
+ if ( globals )
+ {
+ FT_Memory memory = globals->face->memory;
+ FT_UInt nn;
+
+ for ( nn = 0; nn < AF_SCRIPT_MAX; nn++ )
+ {
+ if ( globals->metrics[nn] )
+ {
+ AF_ScriptClass clazz = af_script_classes[nn];
+
+ FT_ASSERT( globals->metrics[nn].clazz == clazz );
+
+ if ( clazz->script_metrics_done )
+ clazz->script_metrics_done( globals->metrics[nn] );
+
+ FT_FREE( globals->metrics[nn] );
+ }
+ }
+
+ globals->glyph_count = NULL;
+ globals->glyph_scripts = NULL; /* no need to free this one !! */
+ globals->face = NULL;
+ FT_FREE( globals );
+ }
+ }
+
+
+ FT_LOCAL_DEF( FT_Error )
+ af_face_globals_get_metrics( AF_FaceGlobals globals,
+ FT_UInt gindex,
+ AF_ScriptMetrics *ametrics )
+ {
+ FT_Script script;
+ AF_ScriptMetrics metrics = NULL;
+ FT_UInt index;
+ AF_ScriptClass clazz;
+ FT_Error error;
+
+ if ( gindex >= globals->glyph_count )
+ {
+ error = FT_Err_Invalid_Argument;
+ goto Exit;
+ }
+
+ index = globals->glyph_scripts[ gindex ];
+ clazz = af_script_class_list[ index ];
+ metrics = globals->metrics[ clazz->script ];
+ if ( metrics == NULL )
+ {
+ /* create the global metrics object when needed
+ */
+ if ( FT_ALLOC( metrics, clazz->script_metrics_size ) )
+ goto Exit;
+
+ metrics->clazz = clazz;
+
+ if ( clazz->script_metrics_init )
+ {
+ error = clazz->script_metrics_init( metrics, face );
+ if ( error )
+ {
+ if ( clazz->script_metrics_done )
+ clazz->script_metrics_done( metrics );
+
+ FT_FREE( metrics );
+ goto Exit;
+ }
+ }
+
+ globals->metrics[ script ] = metrics;
+ }
+
+ Exit:
+ *ametrics = metrics;
+ return error;
+ }
diff --git a/src/autofit/afglobal.h b/src/autofit/afglobal.h
new file mode 100644
index 0000000..81b4800
--- /dev/null
+++ b/src/autofit/afglobal.h
@@ -0,0 +1,42 @@
+#ifndef __AF_GLOBAL_H__
+#define __AF_GLOBAL_H__
+
+#include "aftypes.h"
+
+FT_BEGIN_HEADER
+
+ /**************************************************************************/
+ /**************************************************************************/
+ /***** *****/
+ /***** F A C E G L O B A L S *****/
+ /***** *****/
+ /**************************************************************************/
+ /**************************************************************************/
+
+
+ /*
+ * models the global hints data for a given face, decomposed into
+ * script-specific items..
+ *
+ */
+ typedef struct AF_FaceGlobalsRec_* AF_FaceGlobals;
+
+
+ FT_LOCAL( FT_Error )
+ af_face_globals_new( FT_Face face,
+ AF_FaceGlobals *aglobals );
+
+
+ FT_LOCAL( FT_Error )
+ af_face_globals_get_metrics( AF_FaceGlobals globals,
+ FT_UInt gindex,
+ AF_ScriptMetrics *ametrics );
+
+ FT_LOCAL( void )
+ af_face_globals_free( AF_FaceGlobals globals );
+
+ /* */
+
+FT_END_HEADER
+
+#endif /* __AF_GLOBALS_H__ */
diff --git a/src/autofit/afhints.c b/src/autofit/afhints.c
index b7d8efb..9becdd5 100644
--- a/src/autofit/afhints.c
+++ b/src/autofit/afhints.c
@@ -5,7 +5,7 @@
#include <stdio.h>
void
- af_outline_hints_dump_edges( AF_OutlineHints hints )
+ af_glyph_hints_dump_edges( AF_GlyphHints hints )
{
AF_Edge edges;
AF_Edge edge_limit;
@@ -57,7 +57,7 @@
/* A function used to dump the array of linked segments */
void
- af_outline_hints_dump_segments( AF_OutlineHints hints )
+ af_glyph_hints_dump_segments( AF_GlyphHints hints )
{
AF_Segment segments;
AF_Segment segment_limit;
@@ -139,15 +139,12 @@
/* compute all inflex points in a given glyph */
static void
- af_outline_hints_compute_inflections( AF_OutlineHints hints )
+ af_glyph_hints_compute_inflections( AF_GlyphHints hints )
{
AF_Point* contour = hints->contours;
AF_Point* contour_limit = contour + hints->num_contours;
- /* load original coordinates in (u,v) */
- af_outline_hints_setup_uv( hints, outline, AF_UV_FXY );
-
/* do each contour separately */
for ( ; contour < contour_limit; contour++ )
{
@@ -172,10 +169,10 @@
if ( end == first )
goto Skip;
- } while ( end->u == first->u && end->v == first->v );
+ } while ( end->fx == first->fx && end->fy == first->fy );
- angle_seg = af_angle( end->u - start->u,
- end->v - start->v );
+ angle_seg = af_angle( end->fx - start->fx,
+ end->fy - start->fy );
/* extend the segment start whenever possible */
before = start;
@@ -188,10 +185,10 @@
if ( before == first )
goto Skip;
- } while ( before->u == start->u && before->v == start->v );
+ } while ( before->fx == start->fx && before->fy == start->fy );
- angle_in = af_angle( start->u - before->u,
- start->v - before->v );
+ angle_in = af_angle( start->fx - before->fx,
+ start->fy - before->fy );
} while ( angle_in == angle_seg );
@@ -212,12 +209,12 @@
if ( after == first )
finished = 1;
- } while ( end->u == after->u && end->v == after->v );
+ } while ( end->fx == after->fx && end->fy == after->fy );
- vec.x = after->u - end->u;
- vec.y = after->v - end->v;
- angle_out = af_angle( after->u - end->u,
- after->v - end->v );
+ vec.x = after->fx - end->fx;
+ vec.y = after->fy - end->fy;
+ angle_out = af_angle( after->fx - end->fx,
+ after->fy - end->fy );
} while ( angle_out == angle_seg );
@@ -252,17 +249,17 @@
FT_LOCAL_DEF( void )
- af_outline_hints_init( AF_OutlineHints hints,
- FT_Memory memory )
+ af_glyph_hints_init( AF_GlyphHints hints,
+ FT_Memory memory )
{
FT_ZERO( hints );
hints->memory = memory;
- }
+ }
FT_LOCAL_DEF( void )
- af_outline_hints_done( AF_OutlineHints hints )
+ af_glyph_hints_done( AF_GlyphHints hints )
{
if ( hints && hints->memory )
{
@@ -275,7 +272,7 @@
for ( dim = 0; dim < 2; dim++ )
{
AF_AxisHints axis = &hints->axis[ dim ];
-
+
axis->num_segments = 0;
axis->num_edges = 0;
axis->segments = NULL;
@@ -285,11 +282,11 @@
FT_FREE( hints->contours );
hints->max_contours = 0;
hints->num_contours = 0;
-
+
FT_FREE( hints->points );
hints->num_points = 0;
hints->max_points = 0;
-
+
hints->memory = NULL;
}
}
@@ -297,23 +294,25 @@
FT_LOCAL_DEF( FT_Error )
- af_outline_hints_reset( AF_OutlineHints hints,
- FT_Outline* outline,
- FT_Fixed x_scale,
- FT_Fixed y_scale )
+ af_glyph_hints_reset( AF_GlyphHints hints,
+ FT_Outline* outline,
+ FT_Fixed x_scale,
+ FT_Fixed y_scale,
+ FT_Pos x_delta,
+ FT_Pos y_delta )
{
FT_Error error = AF_Err_Ok;
-
+
FT_UInt old_max, new_max;
hints->num_points = 0;
hints->num_contours = 0;
-
+
hints->axis[0].num_segments = 0;
hints->axis[0].num_edges = 0;
hints->axis[1].num_segments = 0;
hints->axis[1].num_edges = 0;
-
+
/* first of all, reallocate the contours array when necessary
*/
new_max = (FT_UInt) outline->n_contours;
@@ -321,34 +320,34 @@
if ( new_max > old_max )
{
new_max = (new_max + 3) & ~3;
-
+
if ( FT_RENEW_ARRAY( hints->contours, old_max, new_max ) )
goto Exit;
-
+
hints->max_contours = new_max;
}
- /* then, reallocate the points, segments & edges arrays if needed --
- * note that we reserved two additional point positions, used to
- * hint metrics appropriately
- */
+ /* then, reallocate the points, segments & edges arrays if needed --
+ * note that we reserved two additional point positions, used to
+ * hint metrics appropriately
+ */
new_max = (FT_UInt)( outline->n_points + 2 );
old_max = hints->max_points;
if ( new_max > old_max )
{
FT_Byte* items;
FT_ULong off1, off2, off3;
-
+
/* we store in a single buffer the following arrays:
*
* - an array of N AF_PointRec items
* - an array of 2*N AF_SegmentRec items
- * - an array of 2*N AF_EdgeRec items
+ * - an array of 2*N AF_EdgeRec items
*
*/
-
+
new_max = ( new_max + 2 + 7 ) & ~7;
-
+
#undef OFF_INCREMENT
#define OFF_INCREMENT( _off, _type, _count ) \
((((_off) + sizeof(_type)) & ~(sizeof(_type)) + ((_count)*sizeof(_type)))
@@ -368,15 +367,15 @@
hints->axis[1].edges = NULL;
goto Exit;
}
-
+
/* readjust some pointers
*/
hints->max_points = new_max;
hints->points = (AF_Point) items;
-
+
hints->axis[0].segments = (AF_Segment)( items + off1 );
hints->axis[1].segments = hints->axis[0].segments + new_max;
-
+
hints->axis[0].edges = (AF_Edge) ( items + off2 );
hints->axis[1].edges = hints->axis[0].edges + new_max;
}
@@ -422,8 +421,8 @@
{
point->fx = vec->x;
point->fy = vec->y;
- point->ox = point->x = FT_MulFix( vec->x, x_scale );
- point->oy = point->y = FT_MulFix( vec->y, y_scale );
+ point->ox = point->x = FT_MulFix( vec->x, x_scale ) + x_delta;
+ point->oy = point->y = FT_MulFix( vec->y, y_scale ) + y_delta;
switch ( FT_CURVE_TAG( *tag ) )
{
@@ -503,7 +502,7 @@
prev = point->prev;
in_x = point->fx - prev->fx;
in_y = point->fy - prev->fy;
-
+
point->in_dir = af_compute_direction( in_x, in_y );
next = point->next;
@@ -539,88 +538,400 @@
}
/* compute inflection points
- */
- af_outline_hints_compute_inflections( hints );
+ */
+ af_glyph_hints_compute_inflections( hints );
Exit:
return error;
}
+
+
+ /*
+ *
+ * E D G E P O I N T G R I D - F I T T I N G
+ *
+ */
+
+
FT_LOCAL_DEF( void )
- af_outline_hints_setup_uv( AF_OutlineHints hints,
- AF_UV source )
+ af_glyph_hints_align_edge_points( AF_GlyphHints hints,
+ AF_Dimension dim )
{
- AF_Point point = hints->points;
- AF_Point point_limit = point + hints->num_points;
+ AF_AxisHints axis = & hints->axis[ dim ];
+ AF_Edge edges = axis->edges;
+ AF_Edge edge_limit = edges + axis->num_edges;
+ AF_Edge edge;
-
- switch ( source )
+ for ( edge = edges; edge < edge_limit; edge++ )
{
- case AF_UV_FXY:
- for ( ; point < point_limit; point++ )
+ /* move the points of each segment */
+ /* in each edge to the edge's position */
+ AF_Segment seg = edge->first;
+
+
+ do
{
- point->u = point->fx;
- point->v = point->fy;
- }
- break;
+ AF_Point point = seg->first;
+
+
+ for (;;)
+ {
+ if ( dim == AF_DIMENSION_HORZ )
+ {
+ point->x = edge->pos;
+ point->flags |= AF_FLAG_TOUCH_X;
+ }
+ else
+ {
+ point->y = edge->pos;
+ point->flags |= AF_FLAG_TOUCH_Y;
+ }
+
+ if ( point == seg->last )
+ break;
+
+ point = point->next;
+ }
+
+ seg = seg->edge_next;
+
+ } while ( seg != edge->first );
+ }
+ }
+
+
+ /*
+ *
+ * S T R O N G P O I N T I N T E R P O L A T I O N
+ *
+ */
+
+
+ /* hint the strong points -- this is equivalent to the TrueType `IP' */
+ /* hinting instruction */
+ FT_LOCAL_DEF( void )
+ af_glyph_hints_align_strong_points( AF_GlyphHints hints,
+ AF_Dimension dim )
+ {
+ AF_Point points = hints->points;
+ AF_Point point_limit = points + hints->num_points;;
+ AF_AxisHints axis = &hints->axis[dim];
+ AF_Edge edges = axis->edges;
+ AF_Edge edge_limit = edges + axis->num_edges;
+ AF_Flags touch_flag;
+ AF_Point point;
+ AF_Edge edge;
+
+
+ if ( dim == AF_DIMENSION_HORZ )
+ touch_flag = AF_FLAG_TOUCH_X;
+ else
+ touch_flag = AF_FLAG_TOUCH_Y;
+
+ if ( edges < edge_limit )
+ {
+ AF_Point point;
+ AF_Edge edge;
- case AF_UV_FYX:
- for ( ; point < point_limit; point++ )
+ for ( point = points; point < point_limit; point++ )
{
- point->u = point->fy;
- point->v = point->fx;
+ FT_Pos u, ou, fu; /* point position */
+ FT_Pos delta;
+
+
+ if ( point->flags & touch_flag )
+ continue;
+
+ /* if this point is candidate to weak interpolation, we will */
+ /* interpolate it after all strong points have been processed */
+ if ( ( point->flags & AF_FLAG_WEAK_INTERPOLATION ) &&
+ !( point->flags & AF_FLAG_INFLECTION ) )
+ continue;
+
+ if ( dimension )
+ {
+ u = point->fy;
+ ou = point->oy;
+ }
+ else
+ {
+ u = point->fx;
+ ou = point->ox;
+ }
+
+ fu = u;
+
+ /* is the point before the first edge? */
+ edge = edges;
+ delta = edge->fpos - u;
+ if ( delta >= 0 )
+ {
+ u = edge->pos - ( edge->opos - ou );
+ goto Store_Point;
+ }
+
+ /* is the point after the last edge? */
+ edge = edge_limit - 1;
+ delta = u - edge->fpos;
+ if ( delta >= 0 )
+ {
+ u = edge->pos + ( ou - edge->opos );
+ goto Store_Point;
+ }
+
+ {
+ FT_UInt min, max, mid;
+ FT_Pos fpos;
+
+
+ /* find enclosing edges */
+ min = 0;
+ max = edge_limit - edges;
+
+ while ( min < max )
+ {
+ mid = ( max + min ) >> 1;
+ edge = edges + mid;
+ fpos = edge->fpos;
+
+ if ( u < fpos )
+ max = mid;
+ else if ( u > fpos )
+ min = mid + 1;
+ else
+ {
+ /* we are on the edge */
+ u = edge->pos;
+ goto Store_Point;
+ }
+ }
+
+ {
+ AF_Edge before = edges + min - 1;
+ AF_Edge after = edges + min + 0;
+
+
+ /* assert( before && after && before != after ) */
+ if ( before->scale == 0 )
+ before->scale = FT_DivFix( after->pos - before->pos,
+ after->fpos - before->fpos );
+
+ u = before->pos + FT_MulFix( fu - before->fpos,
+ before->scale );
+ }
+ }
+
+
+ Store_Point:
+
+ /* save the point position */
+ if ( dim == AF_DIMENSION_HORZ )
+ point->x = u;
+ else
+ point->y = u;
+
+ point->flags |= touch_flag;
}
- break;
+ }
+ }
+
+
+ /*
+ *
+ * W E A K P O I N T I N T E R P O L A T I O N
+ *
+ */
+
+ static void
+ af_iup_shift( AF_Point p1,
+ AF_Point p2,
+ AF_Point ref )
+ {
+ AF_Point p;
+ FT_Pos delta = ref->u - ref->v;
+
- case AF_UV_OXY:
- for ( ; point < point_limit; point++ )
+ for ( p = p1; p < ref; p++ )
+ p->u = p->v + delta;
+
+ for ( p = ref + 1; p <= p2; p++ )
+ p->u = p->v + delta;
+ }
+
+
+ static void
+ af_iup_interp( AF_Point p1,
+ AF_Point p2,
+ AF_Point ref1,
+ AF_Point ref2 )
+ {
+ AF_Point p;
+ FT_Pos u;
+ FT_Pos v1 = ref1->v;
+ FT_Pos v2 = ref2->v;
+ FT_Pos d1 = ref1->u - v1;
+ FT_Pos d2 = ref2->u - v2;
+
+
+ if ( p1 > p2 )
+ return;
+
+ if ( v1 == v2 )
+ {
+ for ( p = p1; p <= p2; p++ )
{
- point->u = point->ox;
- point->v = point->oy;
+ u = p->v;
+
+ if ( u <= v1 )
+ u += d1;
+ else
+ u += d2;
+
+ p->u = u;
}
- break;
+ return;
+ }
- case AF_UV_OYX:
- for ( ; point < point_limit; point++ )
+ if ( v1 < v2 )
+ {
+ for ( p = p1; p <= p2; p++ )
{
- point->u = point->oy;
- point->v = point->ox;
- }
- break;
+ u = p->v;
- case AF_UV_YX:
- for ( ; point < point_limit; point++ )
+ if ( u <= v1 )
+ u += d1;
+ else if ( u >= v2 )
+ u += d2;
+ else
+ u = ref1->u + FT_MulDiv( u - v1, ref2->u - ref1->u, v2 - v1 );
+
+ p->u = u;
+ }
+ }
+ else
+ {
+ for ( p = p1; p <= p2; p++ )
{
- point->u = point->y;
- point->v = point->x;
+ u = p->v;
+
+ if ( u <= v2 )
+ u += d2;
+ else if ( u >= v1 )
+ u += d1;
+ else
+ u = ref1->u + FT_MulDiv( u - v1, ref2->u - ref1->u, v2 - v1 );
+
+ p->u = u;
}
- break;
+ }
+ }
+
+
+ FT_LOCAL_DEF( void )
+ af_glyph_hints_align_weak_points( AF_GlyphHints hints,
+ AF_Dimension dim )
+ {
+ AF_Point points = hints->points;
+ AF_Point point_limit = points + hints->num_points;
+ AF_Point* contour = hints->contours;
+ AF_Point* contour_limit = contour + hints->num_contours;
+ AF_Flags touch_flag;
+ AF_Point point;
+ AF_Point end_point;
+ AF_Point first_point;
+
- case AF_UV_OX:
- for ( ; point < point_limit; point++ )
+ /* PASS 1: Move segment points to edge positions */
+
+ if ( dim == AF_DIMENSION_HORZ )
+ {
+ touch_flag = AF_FLAG_TOUCH_X;
+
+ for ( point = points; point < point_limit; point++ )
{
point->u = point->x;
point->v = point->ox;
}
- break;
+ }
+ else
+ {
+ touch_flag = AF_FLAG_TOUCH_Y;
- case AF_UV_OY:
- for ( ; point < point_limit; point++ )
+ for ( point = points; point < point_limit; points++ )
{
point->u = point->y;
point->v = point->oy;
}
- break;
+ }
+
+ point = points;
+
+ for ( ; contour < contour_limit; contour++ )
+ {
+ point = *contour;
+ end_point = point->prev;
+ first_point = point;
- default:
- for ( ; point < point_limit; point++ )
+ while ( point <= end_point && !( point->flags & touch_flag ) )
+ point++;
+
+ if ( point <= end_point )
{
- point->u = point->x;
- point->v = point->y;
+ AF_Point first_touched = point;
+ AF_Point cur_touched = point;
+
+
+ point++;
+ while ( point <= end_point )
+ {
+ if ( point->flags & touch_flag )
+ {
+ /* we found two successive touched points; we interpolate */
+ /* all contour points between them */
+ af_iup_interp( cur_touched + 1, point - 1,
+ cur_touched, point );
+ cur_touched = point;
+ }
+ point++;
+ }
+
+ if ( cur_touched == first_touched )
+ {
+ /* this is a special case: only one point was touched in the */
+ /* contour; we thus simply shift the whole contour */
+ af_iup_shift( first_point, end_point, cur_touched );
+ }
+ else
+ {
+ /* now interpolate after the last touched point to the end */
+ /* of the contour */
+ af_iup_interp( cur_touched + 1, end_point,
+ cur_touched, first_touched );
+
+ /* if the first contour point isn't touched, interpolate */
+ /* from the contour start to the first touched point */
+ if ( first_touched > points )
+ af_iup_interp( first_point, first_touched - 1,
+ cur_touched, first_touched );
+ }
}
}
+
+ /* now save the interpolated values back to x/y */
+ if ( dim == AF_DIMENSION_HORZ )
+ {
+ for ( point = points; point < point_limit; point++ )
+ point->x = point->u;
+ }
+ else
+ {
+ for ( point = points; point < point_limit; point++ )
+ point->y = point->u;
+ }
}
+
diff --git a/src/autofit/aflatin.c b/src/autofit/aflatin.c
index f92f67a..f748b11 100644
--- a/src/autofit/aflatin.c
+++ b/src/autofit/aflatin.c
@@ -1,8 +1,438 @@
#include "aflatin.h"
+ /***************************************************************************/
+ /***************************************************************************/
+ /***** *****/
+ /***** L A T I N G L O B A L M E T R I C S *****/
+ /***** *****/
+ /***************************************************************************/
+ /***************************************************************************/
+
+ static void
+ af_latin_metrics_init_widths( AF_LatinMetrics metrics,
+ FT_Face face )
+ {
+ /* scan the array of segments in each direction */
+ AF_GlyphHintsRec hints[1];
+
+ af_glyph_hints_init( hints, face->memory );
+
+ metrics->axis[ AF_DIMENSION_HORZ ].width_count = 0;
+ metrics->axis[ AF_DIMENSION_VERT ].width_count = 0;
+
+ /* For now, compute the standard width and height from the `o' */
+ /* character. I started computing the stem width of the `i' and the */
+ /* stem height of the "-", but it wasn't too good. Moreover, we now */
+ /* have a single character that gives us standard width and height. */
+ {
+ FT_UInt glyph_index;
+ AF_Dimension dim;
+
+
+ glyph_index = FT_Get_Char_Index( face, 'o' );
+ if ( glyph_index == 0 )
+ goto Exit;
+
+ error = FT_Load_Glyph( hinter->face, glyph_index, FT_LOAD_NO_SCALE );
+ if ( error || face->glyph->outline.n_points <= 0 )
+ goto Exit;
+
+ error = af_glyph_hints_reset( hints, &face->glyph->outline,
+ 0x10000L, 0x10000L );
+ if ( error )
+ goto Exit;
+
+ for ( dim = 0; dim < AF_DIMENSION_MAX; dim++ )
+ {
+ AF_LatinAxis axis = & metrics->axis[ dim ];
+ AF_AxisHints axhints = & hints->axis[ dim ];
+ AF_Segment seg, limit, link;
+ FT_UInt num_widths = 0;
+ FT_Pos edge_distance_threshold = 32000;
+
+ af_latin_hints_compute_segments( hints, dim );
+ af_latin_hints_link_segments ( hints, dim );
+
+ seg = axhints->segments;
+ limit = seg + axhints->num_segments;
+
+ for ( ; seg < limit; seg++ )
+ {
+ link = seg->link;
+ /* we only consider stem segments there! */
+ if ( link && link->link == seg && link > seg )
+ {
+ FT_Pos dist;
+
+
+ dist = seg->pos - link->pos;
+ if ( dist < 0 )
+ dist = -dist;
+
+ if ( num_widths < AF_LATIN_MAX_WIDTHS )
+ axis->widths[ num_widths++ ] = dist;
+ }
+ }
+
+ af_sort_pos( axis->widths, num_widths );
+ axis->width_count = num_widths;
+
+ /* we will now try to find the smallest width */
+ if ( num_widths > 0 && axis->widths[0] < edge_distance_threshold )
+ edge_distance_threshold = axis->widths[0];
+
+ /* Now, compute the edge distance threshold as a fraction of the */
+ /* smallest width in the font. Set it in `hinter->glyph' too! */
+ if ( edge_distance_threshold == 32000 )
+ edge_distance_threshold = 50;
+
+ /* let's try 20% */
+ axis->edge_distance_threshold = edge_distance_threshold / 5;
+ }
+ }
+
+ Exit:
+ af_glyph_hints_done( hints );
+ }
+
+
+
+#define AF_LATIN_MAX_TEST_CHARACTERS 12
+
+
+ static const char* const af_latin_blue_chars[ AF_LATIN_MAX_BLUES ] =
+ {
+ "THEZOCQS",
+ "HEZLOCUS",
+ "fijkdbh",
+ "xzroesc",
+ "xzroesc",
+ "pqgjy"
+ };
+
+
+ static void
+ af_latin_metrics_init_blues( AF_LatinMetrics metrics,
+ FT_Face face )
+ {
+ FT_Pos flats [ AF_LATIN_MAX_TEST_CHARACTERS ];
+ FT_Pos rounds[ AF_LATIN_MAX_TEST_CHARACTERS ];
+ FT_Int num_flats;
+ FT_Int num_rounds;
+ FT_Int bb;
+ AF_LatinBlue blue;
+ FT_Error error;
+ AF_LatinAxis axis = &metrics->axis[ AF_DIMENSION_VERT ];
+ FT_GlyphSlot glyph = face->glyph;
+
+ /* we compute the blues simply by loading each character from the */
+ /* 'af_latin_blue_chars[blues]' string, then compute its top-most or */
+ /* bottom-most points (depending on `AF_IS_TOP_BLUE') */
+
+ AF_LOG(( "blue zones computation\n" ));
+ AF_LOG(( "------------------------------------------------\n" ));
+
+ for ( bb = 0; bb < AF_BLUE_MAX; bb++ )
+ {
+ const char* p = af_latin_blue_chars[bb];
+ const char* limit = p + AF_LATIN_MAX_TEST_CHARACTERS;
+ FT_Pos* blue_ref;
+ FT_Pos* blue_shoot;
+
+ AF_LOG(( "blue %3d: ", blue ));
+
+ num_flats = 0;
+ num_rounds = 0;
+
+ for ( ; p < limit && *p; p++ )
+ {
+ FT_UInt glyph_index;
+ FT_Vector* extremum;
+ FT_Vector* points;
+ FT_Vector* point_limit;
+ FT_Vector* point;
+ FT_Bool round;
+
+
+ AF_LOG(( "'%c'", *p ));
+
+ /* load the character in the face -- skip unknown or empty ones */
+ glyph_index = FT_Get_Char_Index( face, (FT_UInt)*p );
+ if ( glyph_index == 0 )
+ continue;
+
+ error = FT_Load_Glyph( face, glyph_index, FT_LOAD_NO_SCALE );
+ if ( error || glyph->outline.n_points <= 0 )
+ continue;
+
+ /* now compute min or max point indices and coordinates */
+ points = glyph->outline.points;
+ point_limit = points + glyph->outline.n_points;
+ point = points;
+ extremum = point;
+ point++;
+
+ if ( AF_IS_TOP_BLUE( bb ) )
+ {
+ for ( ; point < point_limit; point++ )
+ if ( point->y > extremum->y )
+ extremum = point;
+ }
+ else
+ {
+ for ( ; point < point_limit; point++ )
+ if ( point->y < extremum->y )
+ extremum = point;
+ }
+
+ AF_LOG(( "%5d", (int)extremum->y ));
+
+ /* now, check whether the point belongs to a straight or round */
+ /* segment; we first need to find in which contour the extremum */
+ /* lies, then see its previous and next points */
+ {
+ FT_Int idx = (FT_Int)( extremum - points );
+ FT_Int n;
+ FT_Int first, last, prev, next, end;
+ FT_Pos dist;
+
+
+ last = -1;
+ first = 0;
+
+ for ( n = 0; n < glyph->outline.n_contours; n++ )
+ {
+ end = glyph->outline.contours[n];
+ if ( end >= idx )
+ {
+ last = end;
+ break;
+ }
+ first = end + 1;
+ }
+
+ /* XXX: should never happen! */
+ if ( last < 0 )
+ continue;
+
+ /* now look for the previous and next points that are not on the */
+ /* same Y coordinate. Threshold the `closeness'... */
+
+ prev = idx;
+ next = prev;
+
+ do
+ {
+ if ( prev > first )
+ prev--;
+ else
+ prev = last;
+
+ dist = points[prev].y - extremum->y;
+ if ( dist < -5 || dist > 5 )
+ break;
+
+ } while ( prev != idx );
+
+ do
+ {
+ if ( next < last )
+ next++;
+ else
+ next = first;
+
+ dist = points[next].y - extremum->y;
+ if ( dist < -5 || dist > 5 )
+ break;
+
+ } while ( next != idx );
+
+ /* now, set the `round' flag depending on the segment's kind */
+ round = FT_BOOL(
+ FT_CURVE_TAG( glyph->outline.tags[prev] ) != FT_CURVE_TAG_ON ||
+ FT_CURVE_TAG( glyph->outline.tags[next] ) != FT_CURVE_TAG_ON );
+
+ AF_LOG(( "%c ", round ? 'r' : 'f' ));
+ }
+
+ if ( round )
+ rounds[num_rounds++] = extremum->y;
+ else
+ flats[num_flats++] = extremum->y;
+ }
+
+ AF_LOG(( "\n" ));
+
+ if ( num_flats == 0 && num_rounds == 0 )
+ {
+ /* we couldn't find a single glyph to compute this blue zone,
+ * we will simply ignore it then
+ */
+ AF_LOG(( "empty !!\n" ));
+ continue;
+ }
+
+ /* we have computed the contents of the `rounds' and `flats' tables, */
+ /* now determine the reference and overshoot position of the blue -- */
+ /* we simply take the median value after a simple sort */
+ af_sort_pos( num_rounds, rounds );
+ af_sort_pos( num_flats, flats );
+
+ blue = axis->blues[ axis->blue_count ];
+ blue_ref = & blue->ref.org;
+ blue_shoot = & blue->shoot.org;
+
+ axis->blue_count ++;
+
+ if ( num_flats == 0 )
+ {
+ *blue_ref =
+ *blue->shoot = rounds[num_rounds / 2];
+ }
+ else if ( num_rounds == 0 )
+ {
+ *blue_ref =
+ *blue_shoot = flats[num_flats / 2];
+ }
+ else
+ {
+ *blue_ref = flats[num_flats / 2];
+ *blue_shoot = rounds[num_rounds / 2];
+ }
+
+ /* there are sometimes problems: if the overshoot position of top */
+ /* zones is under its reference position, or the opposite for bottom */
+ /* zones. We must thus check everything there and correct the errors */
+ if ( *blue_shoot != *blue_ref )
+ {
+ FT_Pos ref = *blue_ref;
+ FT_Pos shoot = *blue_shoot;
+ FT_Bool over_ref = FT_BOOL( shoot > ref );
+
+
+ if ( AF_LATIN_IS_TOP_BLUE( bb ) ^ over_ref )
+ *blue_shoot = *blue_ref = ( shoot + ref ) / 2;
+ }
+
+ blue->flags = 0;
+ if ( AF_LATIN_IS_TOP_BLUE(bb) )
+ blue->flags |= AF_LATIN_BLUE_TOP;
+
+ AF_LOG(( "-- ref = %ld, shoot = %ld\n", *blue_ref, *blue_shoot ));
+ }
+
+ Exit:
+ return;
+ }
+
+
+ static FT_Error
+ af_latin_metrics_init( AF_LatinMetrics metrics,
+ FT_Face face )
+ {
+ FT_Error error;
+ FT_CharMap oldmap = face->charmap;
+
+ /* do we have a Unicode charmap in there? */
+ error = FT_Select_Charmap( face, FT_ENCODING_UNICODE );
+ if ( error ) goto Exit;
+
+ metrics->units_per_em = face->units_per_EM;
+
+ af_latin_metrics_init_widths( metrics, face );
+ af_latin_metrics_init_blues( metrics, face );
+
+ Exit:
+ FT_Set_Charmap( face, oldmap );
+ return error;
+ }
+
+
+ static void
+ af_latin_metrics_scale_dim( AF_LatinMetrics metrics,
+ AF_Scaler scaler,
+ AF_Dimension dim )
+ {
+ FT_Fixed scale;
+ FT_Pos delta;
+ AF_LatinAxis axis;
+ FT_UInt nn;
+
+ if ( dim == AF_DIMENSION_HORZ )
+ {
+ scale = scaler->x_scale;
+ delta = scaler->x_delta;
+ }
+ else
+ {
+ scale = scaler->y_scale;
+ delta = scaler->y_delta;
+ }
+
+ axis = metrics->axis[ dim ];
+
+ if ( axis->scale == scale && axis->delta == delta )
+ return;
+
+ axis->scale = scale;
+ axis->delta = delta;
+
+ /* scale the standard widths
+ */
+ for ( nn = 0; nn < axis->width_count; nn++ )
+ {
+ AF_Width width = axis->widths + nn;
+
+ width->cur = FT_MulFix( width->org, scale );
+ width->fit = width->cur;
+ }
+
+ if ( dim == AF_DIMENSION_VERT )
+ {
+ /* scale the blue zones
+ */
+ for ( nn = 0; nn < axis->blue_count; nn++ )
+ {
+ AF_LatinBlue blue = & axis->blues[nn];
+ FT_UInt flags = blue->flags & ~AF_LATIN_BLUE_ACTIVE;
+ FT_Pos dist;
+
+ blue->ref.cur = FT_MulFix( blue->ref.org, scale ) + delta;
+ blue->ref.fit = blue->ref.cur;
+
+ blue->shoot.cur = FT_MulFix( blue->shoot.org, scale ) + delta;
+ blue->shoot.fit = blue->shoot.cur;
+
+ /* a blue zone is only active when it is less than 3/4 pixels tall
+ */
+ dist = FT_MulFix( blue->ref.org - blue->shoot.org, scale );
+ if ( dist >= 48 || dist <= -48 )
+ blue->flags |= ~AF_LATIN_BLUE_ACTIVE;
+ }
+ }
+ }
+
+
FT_LOCAL_DEF( void )
- af_latin_hints_compute_segments( AF_OutlineHints hints,
- AF_Dimension dim )
+ af_latin_metrics_scale( AF_LatinMetrics metrics,
+ AF_Scaler scaler )
+ {
+ af_latin_metrics_scale( metrics, scaler, AF_DIMENSION_HORZ );
+ af_latin_metrics_scale( metrics, scaler, AF_DIMENSION_VERT );
+ }
+
+
+ /***************************************************************************/
+ /***************************************************************************/
+ /***** *****/
+ /***** L A T I N G L Y P H A N A L Y S I S *****/
+ /***** *****/
+ /***************************************************************************/
+ /***************************************************************************/
+
+ FT_LOCAL_DEF( void )
+ af_latin_hints_compute_segments( AF_GlyphHints hints,
+ AF_Dimension dim )
{
AF_AxisHints axis = &hints->axis[dim];
AF_Segment segments = axis->segments;
@@ -23,9 +453,28 @@
segment_dir = major_dir;
/* set up (u,v) in each point */
- af_setup_uv( outline, (dim == AF_DIMENSION_HORZ)
- ? AF_UV_FXY,
- : AF_UV_FYX );
+ if ( dim == AF_DIMENSION_HORZ )
+ {
+ AF_Point point = hints->points;
+ AF_Point limit = point + hints->num_points;
+
+ for ( ; point < limit; point++ )
+ {
+ point->u = point->fx;
+ point->v = point->fy;
+ }
+ }
+ else
+ {
+ AF_Point point = hints->points;
+ AF_Point limit = point + hints->num_points;
+
+ for ( ; point < limit; point++ )
+ {
+ point->u = point->fy;
+ point->v = point->fx;
+ }
+ }
/* do each contour separately */
@@ -238,8 +687,8 @@
FT_LOCAL_DEF( void )
- af_latin_hints_link_segments( AF_OutlineHints hints,
- AF_Dimension dim )
+ af_latin_hints_link_segments( AF_GlyphHints hints,
+ AF_Dimension dim )
{
AF_AxisHints axis = &hints->axis[dim];
AF_Segment segments = axis->segments;
@@ -318,17 +767,17 @@
FT_LOCAL_DEF( void )
- af_latin_hints_compute_edges( AF_OutlineHints hints,
- AF_Dimension dim )
+ af_latin_hints_compute_edges( AF_GlyphHints hints,
+ AF_Dimension dim )
{
AF_AxisHints axis = &hints->axis[dim];
AF_Edge edges = axis->edges;
AF_Edge edge, edge_limit;
-
+
AF_Segment segments = axis->segments;
AF_Segment segment_limit = segments + axis->num_segments;
AF_Segment seg;
-
+
AF_Direction up_dir;
FT_Fixed scale;
FT_Pos edge_distance_threshold;
@@ -519,7 +968,6 @@
else
edge2 = seg2->edge;
-#ifdef FT_CONFIG_CHESTER_SERIF
if ( is_serif )
{
edge->serif = edge2;
@@ -527,12 +975,6 @@
}
else
edge->link = edge2;
-#else /* !FT_CONFIG_CHESTER_SERIF */
- if ( is_serif )
- edge->serif = edge2;
- else
- edge->link = edge2;
-#endif /* !FT_CONFIG_CHESTER_SERIF */
}
seg = seg->edge_next;
@@ -567,17 +1009,9 @@
}
- /*************************************************************************/
- /* */
- /* <Function> */
- /* af_outline_detect_features */
- /* */
- /* <Description> */
- /* Performs feature detection on a given AF_OutlineRec object. */
- /* */
FT_LOCAL_DEF( void )
- af_latin_hints_detect_features( AF_OutlineHints hints,
- AF_Dimension dim )
+ af_latin_hints_detect_features( AF_GlyphHints hints,
+ AF_Dimension dim )
{
af_latin_hints_compute_segments( hints, dim );
af_latin_hints_link_segments ( hints, dim );
@@ -585,92 +1019,50 @@
}
- /*************************************************************************/
- /* */
- /* <Function> */
- /* af_outline_compute_blue_edges */
- /* */
- /* <Description> */
- /* Computes the `blue edges' in a given outline (i.e. those that must */
- /* be snapped to a blue zone edge (top or bottom). */
- /* */
FT_LOCAL_DEF( void )
- af_latin_hints_compute_blue_edges( AF_OutlineHints outline,
- AF_Face_Globals face_globals )
+ af_latin_hints_compute_blue_edges( AF_GlyphHints hints,
+ AF_LatinMetrics metrics )
{
- AF_Edge edge = outline->horz_edges;
- AF_Edge edge_limit = edge + outline->num_hedges;
- AF_Globals globals = &face_globals->design;
- FT_Fixed y_scale = outline->y_scale;
-
- FT_Bool blue_active[AF_BLUE_MAX];
+ AF_AxisHints axis = &hints->axis[ AF_DIMENSION_VERT ];
+ AF_Edge edge = axis->edges;
+ AF_Edge edge_limit = edge + axis->num_edges;
+ AF_LatinAxis latin = &metrics->axis[ AF_DIMENSION_VERT ];
+ FT_Fixed scale = latin->scale;
/* compute which blue zones are active, i.e. have their scaled */
/* size < 3/4 pixels */
- {
- AF_Blue blue;
- FT_Bool check = 0;
-
- for ( blue = AF_BLUE_CAPITAL_TOP; blue < AF_BLUE_MAX; blue++ )
- {
- FT_Pos ref, shoot, dist;
-
-
- ref = globals->blue_refs[blue];
- shoot = globals->blue_shoots[blue];
- dist = ref - shoot;
- if ( dist < 0 )
- dist = -dist;
-
- blue_active[blue] = 0;
-
- if ( FT_MulFix( dist, y_scale ) < 48 )
- {
- blue_active[blue] = 1;
- check = 1;
- }
- }
-
- /* return immediately if no blue zone is active */
- if ( !check )
- return;
- }
-
- /* for each horizontal edge search the blue zone which is closest */
+ /* for each horizontal edge search the blue zone which is closest */
for ( ; edge < edge_limit; edge++ )
{
- AF_Blue blue;
- FT_Pos* best_blue = 0;
- FT_Pos best_dist; /* initial threshold */
+ FT_Int bb;
+ AF_Width best_blue = NULL;
+ FT_Pos best_dist; /* initial threshold */
/* compute the initial threshold as a fraction of the EM size */
- best_dist = FT_MulFix( face_globals->face->units_per_EM / 40, y_scale );
+ best_dist = FT_MulFix( metrics->units_pe_EM / 40, scale );
-#ifdef FT_CONFIG_CHESTER_SMALL_F
if ( best_dist > 64 / 2 )
best_dist = 64 / 2;
-#else
- if ( best_dist > 64 / 4 )
- best_dist = 64 / 4;
-#endif
- for ( blue = AF_BLUE_CAPITAL_TOP; blue < AF_BLUE_MAX; blue++ )
+ for ( bb = 0; bb < AF_LATIN_BLUE_MAX; bb++ )
{
+ AF_LatinBlue blue = latin->blues + bb;
+ FT_Bool is_top_blue, is_major_dir;
+
+ /* skip inactive blue zones (i.e. those that are too small
+ */
+ if ( !(bb->flags & AF_LATIN_BLUE_ACTIVE) )
+ continue;
+
/* if it is a top zone, check for right edges -- if it is a bottom */
/* zone, check for left edges */
/* */
/* of course, that's for TrueType XXX */
- FT_Bool is_top_blue =
- FT_BOOL( AF_IS_TOP_BLUE( blue ) );
- FT_Bool is_major_dir =
- FT_BOOL( edge->dir == outline->horz_major_dir );
-
-
- if ( !blue_active[blue] )
- continue;
+ is_top_blue = (bb->flags & AF_LATIN_BLUE_TOP) != 0;
+ is_major_dir = FT_BOOL( edge->dir == axis->major_dir );
/* if it is a top zone, the edge must be against the major */
/* direction; if it is a bottom zone, it must be in the major */
@@ -678,11 +1070,10 @@
if ( is_top_blue ^ is_major_dir )
{
FT_Pos dist;
- FT_Pos* blue_pos = globals->blue_refs + blue;
/* first of all, compare it to the reference position */
- dist = edge->fpos - *blue_pos;
+ dist = edge->fpos - blue->ref.org;
if ( dist < 0 )
dist = -dist;
@@ -690,7 +1081,7 @@
if ( dist < best_dist )
{
best_dist = dist;
- best_blue = blue_pos;
+ best_blue = & blue->ref;
}
/* now, compare it to the overshoot position if the edge is */
@@ -698,13 +1089,13 @@
/* top zone, or under the reference position of a bottom zone */
if ( edge->flags & AF_EDGE_ROUND && dist != 0 )
{
- FT_Bool is_under_ref = FT_BOOL( edge->fpos < *blue_pos );
+ FT_Bool is_under_ref = FT_BOOL( edge->fpos < blue->ref.org );
if ( is_top_blue ^ is_under_ref )
{
- blue_pos = globals->blue_shoots + blue;
- dist = edge->fpos - *blue_pos;
+ blue = _pos = globals->blue_shoots + blue;
+ dist = edge->fpos - blue->shoot.org;
if ( dist < 0 )
dist = -dist;
@@ -712,7 +1103,7 @@
if ( dist < best_dist )
{
best_dist = dist;
- best_blue = blue_pos;
+ best_blue = & blue->shoot;
}
}
}
@@ -725,31 +1116,600 @@
}
+ static FT_Error
+ af_latin_hints_init( AF_GlyphHints hints,
+ AF_Scaler scaler,
+ AF_LatinMetrics metrics )
+ {
+ FT_Error error;
+
+ error = af_glyph_hints_reset( hints, &scaler->outline,
+ scaler->x_scale,
+ scaler->y_scale,
+ scaler->x_delta,
+ scaler->y_delta );
+ if (error)
+ goto Exit;
+
+ af_latin_hints_detect_features( hints, metrics );
+ af_latin_hints_compute_blue_edges( hints, metrics );
+
+ Exit:
+ return error;
+ }
+
+ /***************************************************************************/
+ /***************************************************************************/
+ /***** *****/
+ /***** L A T I N G L Y P H G R I D - F I T T I N G *****/
+ /***** *****/
+ /***************************************************************************/
+ /***************************************************************************/
+
+ /* snap a given width in scaled coordinates to one of the */
+ /* current standard widths */
+ static FT_Pos
+ af_latin_snap_width( AF_Width widths,
+ FT_Int count,
+ FT_Pos width )
+ {
+ int n;
+ FT_Pos best = 64 + 32 + 2;
+ FT_Pos reference = width;
+ FT_Pos scaled;
+
+
+ for ( n = 0; n < count; n++ )
+ {
+ FT_Pos w;
+ FT_Pos dist;
+
+
+ w = widths[n].cur;
+ dist = width - w;
+ if ( dist < 0 )
+ dist = -dist;
+ if ( dist < best )
+ {
+ best = dist;
+ reference = w;
+ }
+ }
+
+ scaled = ( reference + 32 ) & -64;
+
+ if ( width >= reference )
+ {
+ if ( width < scaled + 48 )
+ width = reference;
+ }
+ else
+ {
+ if ( width > scaled - 48 )
+ width = reference;
+ }
+
+ return width;
+ }
+
+
+ /* compute the snapped width of a given stem */
+
+ static FT_Pos
+ af_latin_compute_stem_width( AF_GylphHints hints,
+ AF_Dimension dim,
+ FT_Pos width,
+ AF_Edge_Flags base_flags,
+ AF_Edge_Flags stem_flags )
+ {
+ AF_Globals globals = &hinter->globals->scaled;
+ FT_Pos dist = width;
+ FT_Int sign = 0;
+ FT_Bool vertical = FT_BOOL( dim == AF_DIMENSION_VERT );
+
+
+ if ( dist < 0 )
+ {
+ dist = -width;
+ sign = 1;
+ }
+
+ if ( !hinter->do_stem_adjust )
+ {
+ /* leave stem widths unchanged */
+ }
+ else if ( ( vertical && !hinter->do_vert_snapping ) ||
+ ( !vertical && !hinter->do_horz_snapping ) )
+ {
+ /* smooth hinting process: very lightly quantize the stem width */
+ /* */
+
+ /* leave the widths of serifs alone */
+
+ if ( ( stem_flags & AF_EDGE_SERIF ) && vertical && ( dist < 3 * 64 ) )
+ goto Done_Width;
+
+ else if ( ( base_flags & AF_EDGE_ROUND ) )
+ {
+ if ( dist < 80 )
+ dist = 64;
+ }
+ else if ( dist < 56 )
+ dist = 56;
+
+ {
+ FT_Pos delta = dist - globals->stds[vertical];
+
+
+ if ( delta < 0 )
+ delta = -delta;
+
+ if ( delta < 40 )
+ {
+ dist = globals->stds[vertical];
+ if ( dist < 48 )
+ dist = 48;
+
+ goto Done_Width;
+ }
+
+ if ( dist < 3 * 64 )
+ {
+ delta = dist & 63;
+ dist &= -64;
+
+ if ( delta < 10 )
+ dist += delta;
+
+ else if ( delta < 32 )
+ dist += 10;
+
+ else if ( delta < 54 )
+ dist += 54;
+
+ else
+ dist += delta;
+ }
+ else
+ dist = ( dist + 32 ) & -64;
+ }
+ }
+ else
+ {
+ /* strong hinting process: snap the stem width to integer pixels */
+ /* */
+ if ( vertical )
+ {
+ dist = af_snap_width( globals->heights, globals->num_heights, dist );
+
+ /* in the case of vertical hinting, always round */
+ /* the stem heights to integer pixels */
+ if ( dist >= 64 )
+ dist = ( dist + 16 ) & -64;
+ else
+ dist = 64;
+ }
+ else
+ {
+ dist = af_snap_width( globals->widths, globals->num_widths, dist );
+
+ if ( hinter->flags & AF_HINTER_MONOCHROME )
+ {
+ /* monochrome horizontal hinting: snap widths to integer pixels */
+ /* with a different threshold */
+ if ( dist < 64 )
+ dist = 64;
+ else
+ dist = ( dist + 32 ) & -64;
+ }
+ else
+ {
+ /* for horizontal anti-aliased hinting, we adopt a more subtle */
+ /* approach: we strengthen small stems, round stems whose size */
+ /* is between 1 and 2 pixels to an integer, otherwise nothing */
+ if ( dist < 48 )
+ dist = ( dist + 64 ) >> 1;
+
+ else if ( dist < 128 )
+ dist = ( dist + 22 ) & -64;
+ else
+ /* XXX: round otherwise to prevent color fringes in LCD mode */
+ dist = ( dist + 32 ) & -64;
+ }
+ }
+ }
+
+ Done_Width:
+ if ( sign )
+ dist = -dist;
+
+ return dist;
+ }
+
+
+
+ /* align one stem edge relative to the previous stem edge */
+ static void
+ af_latin_align_linked_edge( AF_GlyphHints hints,
+ AF_Dimension dim,
+ AF_Edge base_edge,
+ AF_Edge stem_edge )
+ {
+ FT_Pos dist = stem_edge->opos - base_edge->opos;
+
+ FT_Pos fitted_width = af_latin_compute_stem_width( hints,
+ dim,
+ dist,
+ base_edge->flags,
+ stem_edge->flags );
+
+ stem_edge->pos = base_edge->pos + fitted_width;
+ }
+
+
+ static void
+ af_latin_align_serif_edge( AF_GlyphHints hints,
+ AF_Edge base,
+ AF_Edge serif )
+ {
+ FT_UNUSED( hints );
+
+ serif->pos = base->pos + (serif->opos - base->opos);
+ }
+
+ /*************************************************************************/
+ /*************************************************************************/
+ /*************************************************************************/
+ /**** ****/
+ /**** E D G E H I N T I N G ****/
+ /**** ****/
+ /*************************************************************************/
/*************************************************************************/
- /* */
- /* <Function> */
- /* af_outline_scale_blue_edges */
- /* */
- /* <Description> */
- /* This function must be called before hinting in order to re-adjust */
- /* the contents of the detected edges (basically change the `blue */
- /* edge' pointer from `design units' to `scaled ones'). */
- /* */
+ /*************************************************************************/
+
+
FT_LOCAL_DEF( void )
- af_outline_hints_scale_blue_edges( AF_OutlineHints hints ) outline,
+ af_latin_hint_edges( AF_GlyphHints hints,
+ AF_Dimension dim )
{
- AF_AxisHints axis = &hints->axis[ AF_DIMENSION_VERT ];
- AF_Edge edge = axis->edges;
- AF_Edge edge_limit = edge + axis->num_edges;
- FT_Pos delta;
+ AF_AxisHints axis = & hints->axis[dim];
+ AF_Edge edges = axis->edges;
+ AF_Edge edge_limit = edges + axis->num_edges;
+ FT_Int n_edges;
+ AF_Edge edge;
+ AF_Edge anchor = 0;
+ FT_Int has_serifs = 0;
- delta = globals->scaled.blue_refs - globals->design.blue_refs;
+ /* we begin by aligning all stems relative to the blue zone */
+ /* if needed -- that's only for horizontal edges */
+ if ( dim == AF_DIMENSION_VERT )
+ {
+ for ( edge = edges; edge < edge_limit; edge++ )
+ {
+ AF_Width* blue;
+ AF_Edge edge1, edge2;
- for ( ; edge < edge_limit; edge++ )
+
+ if ( edge->flags & AF_EDGE_DONE )
+ continue;
+
+ blue = edge->blue_edge;
+ edge1 = NULL;
+ edge2 = edge->link;
+
+ if ( blue )
+ {
+ edge1 = edge;
+ }
+ else if ( edge2 && edge2->blue_edge )
+ {
+ blue = edge2->blue_edge;
+ edge1 = edge2;
+ edge2 = edge;
+ }
+
+ if ( !edge1 )
+ continue;
+
+ edge1->pos = blue->fit;
+ edge1->flags |= AF_EDGE_DONE;
+
+ if ( edge2 && !edge2->blue_edge )
+ {
+ af_latin_align_linked_edge( hints, dim, edge1, edge2 );
+ edge2->flags |= AF_EDGE_DONE;
+ }
+
+ if ( !anchor )
+ anchor = edge;
+ }
+ }
+
+ /* now we will align all stem edges, trying to maintain the */
+ /* relative order of stems in the glyph */
+ for ( edge = edges; edge < edge_limit; edge++ )
+ {
+ AF_Edge edge2;
+
+
+ if ( edge->flags & AF_EDGE_DONE )
+ continue;
+
+ /* skip all non-stem edges */
+ edge2 = edge->link;
+ if ( !edge2 )
+ {
+ has_serifs++;
+ continue;
+ }
+
+ /* now align the stem */
+
+ /* this should not happen, but it's better to be safe */
+ if ( edge2->blue_edge || edge2 < edge )
+ {
+ af_latin_align_linked_edge( hinter, edge2, edge, dimension );
+ edge->flags |= AF_EDGE_DONE;
+ continue;
+ }
+
+ if ( !anchor )
+ {
+ FT_Pos org_len, org_center, cur_len;
+ FT_Pos cur_pos1, error1, error2, u_off, d_off;
+
+
+ org_len = edge2->opos - edge->opos;
+ cur_len = af_latin_compute_stem_width( hints, dim, org_len,
+ edge->flags, edge2->flags );
+ if ( cur_len <= 64 )
+ u_off = d_off = 32;
+ else
+ {
+ u_off = 38;
+ d_off = 26;
+ }
+
+ if ( cur_len < 96 )
+ {
+ org_center = edge->opos + ( org_len >> 1 );
+
+ cur_pos1 = ( org_center + 32 ) & -64;
+
+ error1 = org_center - ( cur_pos1 - u_off );
+ if ( error1 < 0 )
+ error1 = -error1;
+
+ error2 = org_center - ( cur_pos1 + d_off );
+ if ( error2 < 0 )
+ error2 = -error2;
+
+ if ( error1 < error2 )
+ cur_pos1 -= u_off;
+ else
+ cur_pos1 += d_off;
+
+ edge->pos = cur_pos1 - cur_len / 2;
+ edge2->pos = cur_pos1 + cur_len / 2;
+
+ }
+ else
+ edge->pos = ( edge->opos + 32 ) & -64;
+
+ anchor = edge;
+
+ edge->flags |= AF_EDGE_DONE;
+
+ af_latin_align_linked_edge( hints, dim, edge, edge2 );
+ }
+ else
+ {
+ FT_Pos org_pos, org_len, org_center, cur_len;
+ FT_Pos cur_pos1, cur_pos2, delta1, delta2;
+
+
+ org_pos = anchor->pos + ( edge->opos - anchor->opos );
+ org_len = edge2->opos - edge->opos;
+ org_center = org_pos + ( org_len >> 1 );
+
+ cur_len = af_compute_stem_width( hinter, dimension, org_len,
+ edge->flags, edge2->flags );
+
+ if ( cur_len < 96 )
+ {
+ FT_Pos u_off, d_off;
+
+
+ cur_pos1 = ( org_center + 32 ) & -64;
+
+ if (cur_len <= 64 )
+ u_off = d_off = 32;
+ else
+ {
+ u_off = 38;
+ d_off = 26;
+ }
+
+ delta1 = org_center - ( cur_pos1 - u_off );
+ if ( delta1 < 0 )
+ delta1 = -delta1;
+
+ delta2 = org_center - ( cur_pos1 + d_off );
+ if ( delta2 < 0 )
+ delta2 = -delta2;
+
+ if ( delta1 < delta2 )
+ cur_pos1 -= u_off;
+ else
+ cur_pos1 += d_off;
+
+ edge->pos = cur_pos1 - cur_len / 2;
+ edge2->pos = cur_pos1 + cur_len / 2;
+ }
+ else
+ {
+ org_pos = anchor->pos + ( edge->opos - anchor->opos );
+ org_len = edge2->opos - edge->opos;
+ org_center = org_pos + ( org_len >> 1 );
+
+ cur_len = af_compute_stem_width( hinter, dimension, org_len,
+ edge->flags, edge2->flags );
+
+ cur_pos1 = ( org_pos + 32 ) & -64;
+ delta1 = ( cur_pos1 + ( cur_len >> 1 ) - org_center );
+ if ( delta1 < 0 )
+ delta1 = -delta1;
+
+ cur_pos2 = ( ( org_pos + org_len + 32 ) & -64 ) - cur_len;
+ delta2 = ( cur_pos2 + ( cur_len >> 1 ) - org_center );
+ if ( delta2 < 0 )
+ delta2 = -delta2;
+
+ edge->pos = ( delta1 < delta2 ) ? cur_pos1 : cur_pos2;
+ edge2->pos = edge->pos + cur_len;
+ }
+
+ edge->flags |= AF_EDGE_DONE;
+ edge2->flags |= AF_EDGE_DONE;
+
+ if ( edge > edges && edge->pos < edge[-1].pos )
+ edge->pos = edge[-1].pos;
+ }
+ }
+
+ /* make sure that lowercase m's maintain their symmetry */
+
+ /* In general, lowercase m's have six vertical edges if they are sans */
+ /* serif, or twelve if they are avec serif. This implementation is */
+ /* based on that assumption, and seems to work very well with most */
+ /* faces. However, if for a certain face this assumption is not */
+ /* true, the m is just rendered like before. In addition, any stem */
+ /* correction will only be applied to symmetrical glyphs (even if the */
+ /* glyph is not an m), so the potential for unwanted distortion is */
+ /* relatively low. */
+
+ /* We don't handle horizontal edges since we can't easily assure that */
+ /* the third (lowest) stem aligns with the base line; it might end up */
+ /* one pixel higher or lower. */
+
+ n_edges = edge_limit - edges;
+ if ( dim == AF_DIMENSION_HORZ && ( n_edges == 6 || n_edges == 12 ) )
+ {
+ AF_Edge edge1, edge2, edge3;
+ FT_Pos dist1, dist2, span, delta;
+
+
+ if ( n_edges == 6 )
+ {
+ edge1 = edges;
+ edge2 = edges + 2;
+ edge3 = edges + 4;
+ }
+ else
+ {
+ edge1 = edges + 1;
+ edge2 = edges + 5;
+ edge3 = edges + 9;
+ }
+
+ dist1 = edge2->opos - edge1->opos;
+ dist2 = edge3->opos - edge2->opos;
+
+ span = dist1 - dist2;
+ if ( span < 0 )
+ span = -span;
+
+ if ( span < 8 )
+ {
+ delta = edge3->pos - ( 2 * edge2->pos - edge1->pos );
+ edge3->pos -= delta;
+ if ( edge3->link )
+ edge3->link->pos -= delta;
+
+ /* move the serifs along with the stem */
+ if ( n_edges == 12 )
+ {
+ ( edges + 8 )->pos -= delta;
+ ( edges + 11 )->pos -= delta;
+ }
+
+ edge3->flags |= AF_EDGE_DONE;
+ if ( edge3->link )
+ edge3->link->flags |= AF_EDGE_DONE;
+ }
+ }
+
+ if ( has_serifs || !anchor )
{
- if ( edge->blue_edge )
- edge->blue_edge += delta;
+ /* now hint the remaining edges (serifs and single) in order
+ * to complete our processing
+ */
+ for ( edge = edges; edge < edge_limit; edge++ )
+ {
+ if ( edge->flags & AF_EDGE_DONE )
+ continue;
+
+ if ( edge->serif )
+ af_align_serif_edge( hinter, edge->serif, edge, dimension );
+ else if ( !anchor )
+ {
+ edge->pos = ( edge->opos + 32 ) & -64;
+ anchor = edge;
+ }
+ else
+ edge->pos = anchor->pos +
+ ( ( edge->opos-anchor->opos + 32 ) & -64 );
+
+ edge->flags |= AF_EDGE_DONE;
+
+ if ( edge > edges && edge->pos < edge[-1].pos )
+ edge->pos = edge[-1].pos;
+
+ if ( edge + 1 < edge_limit &&
+ edge[1].flags & AF_EDGE_DONE &&
+ edge->pos > edge[1].pos )
+ edge->pos = edge[1].pos;
+ }
}
}
+
+ static FT_Error
+ af_latin_hints_apply( AF_GlyphHints hints,
+ AF_Scaler scaler,
+ AF_LatinMetrics metrics )
+ {
+ /* XXX */
+ return FT_Err_Unimplemented;
+ }
+
+ /***************************************************************************/
+ /***************************************************************************/
+ /***** *****/
+ /***** L A T I N S C R I P T C L A S S *****/
+ /***** *****/
+ /***************************************************************************/
+ /***************************************************************************/
+
+ static const AF_Script_UniRangeRec af_latin_uniranges[] =
+ {
+ { 32, 127 }, /* XXX: TODO: Add new Unicode ranges here !! */
+ { 160, 255 },
+ { 0, 0 }
+ };
+
+ FT_LOCAL_DEF( const AF_ScriptClassRec ) af_latin_script_class =
+ {
+ AF_SCRIPT_LATIN,
+ &af_latin_script_uniranges,
+
+ sizeof( AF_LatinMetricsRec ),
+ (AF_Script_InitMetricsFunc) af_latin_metrics_init,
+ (AF_Script_ScaleMetricsFunc) af_latin_metrics_scale,
+ (AF_Script_DoneMetricsFunc) NULL,
+
+ (AF_Script_InitHintsFunc) af_latin_hints_init,
+ (AF_Script_ApplyHintsFunc) af_latin_hints_apply
+ };
+
diff --git a/src/autofit/aflatin.h b/src/autofit/aflatin.h
index 5c716e8..816c5c5 100644
--- a/src/autofit/aflatin.h
+++ b/src/autofit/aflatin.h
@@ -11,6 +11,14 @@ FT_BEGIN_HEADER
*/
FT_LOCAL( const FT_ScriptClassRec ) af_latin_script_class;
+ /***************************************************************************/
+ /***************************************************************************/
+ /***** *****/
+ /***** L A T I N G L O B A L M E T R I C S *****/
+ /***** *****/
+ /***************************************************************************/
+ /***************************************************************************/
+
/*
* the following declarations could be embedded in the file "aflatin.c"
* they've been made semi-public to allow alternate script hinters to
@@ -21,29 +29,65 @@ FT_BEGIN_HEADER
* Latin (global) metrics management
*
*/
-
+
+ enum
+ {
+ AF_LATIN_BLUE_CAPITAL_TOP,
+ AF_LATIN_BLUE_CAPITAL_BOTTOM,
+ AF_LATIN_BLUE_SMALL_F_TOP,
+ AF_LATIN_BLUE_SMALL_TOP,
+ AF_LATIN_BLUE_SMALL_BOTTOM,
+ AF_LATIN_BLUE_SMALL_MINOR,
+
+ AF_LATIN_BLUE_MAX
+ };
+
+#define AF_LATIN_IS_TOP_BLUE( b ) ( (b) == AF_LATIN_BLUE_CAPITAL_TOP || \
+ (b) == AF_LATIN_BLUE_SMALL_F_TOP || \
+ (b) == AF_LATIN_BLUE_SMALL_TOP )
+
#define AF_LATIN_MAX_WIDTHS 16
-#define AF_LATIN_MAX_BLUES 32
+#define AF_LATIN_MAX_BLUES AF_LATIN_BLUE_MAX
+
+ enum
+ {
+ AF_LATIN_BLUE_ACTIVE = (1 << 0),
+ AF_LATIN_BLUE_TOP = (1 << 1),
+
+ AF_LATIN_BLUE_MAX
+ };
+
+
+ typedef struct AF_LatinBlueRec_
+ {
+ AF_WidthRec ref;
+ AF_WidthRec shoot;
+ FT_UInt flags;
+
+ } AF_LatinBlueRec, *AF_LatinBlue;
+
typedef struct AF_LatinAxisRec_
{
- FT_Fixed scale;
- FT_Pos delta;
+ FT_Fixed scale;
+ FT_Pos delta;
- FT_UInt width_count;
- AF_WidthRec widths[ AF_LATIN_MAX_WIDTHS ];
+ FT_UInt width_count;
+ AF_WidthRec widths[ AF_LATIN_MAX_WIDTHS ];
+ FT_Pos edge_distance_threshold;
/* ignored for horizontal metrics */
- FT_Bool control_overshoot;
- FT_UInt blue_count;
- AF_WidthRec blue_refs [ AF_MAX_BLUES ];
- AF_WidthRec blue_shoots[ AF_MAX_BLUES ];
-
+ FT_Bool control_overshoot;
+ FT_UInt blue_count;
+ AF_LatinBlueRec blues;
+
} AF_LatinAxisRec, *AF_LatinAxis;
-
+
+
typedef struct AF_LatinMetricsRec_
{
- AF_OutlineMetricsRec root;
+ AF_ScriptMetricsRec root;
+ FT_UInt units_per_em;
AF_LatinAxisRec axis[ AF_DIMENSION_MAX ];
} AF_LatinMetricsRec, *AF_LatinMetrics;
@@ -58,29 +102,40 @@ FT_BEGIN_HEADER
AF_Scaler scaler );
- /*
- * Latin (glyph) hints management
- *
- */
- FT_LOCAL(
+ /***************************************************************************/
+ /***************************************************************************/
+ /***** *****/
+ /***** L A T I N G L Y P H A N A L Y S I S *****/
+ /***** *****/
+ /***************************************************************************/
+ /***************************************************************************/
- FT_LOCAL( void )
- af_latin_hints_compute_segments( AF_OutlineHints hints,
- AF_Dimension dim );
+ /* this shouldn't normally be exported. However, other scripts might
+ * like to use this function as-is
+ */
FT_LOCAL( void )
- af_latin_hints_link_segments( AF_OutlineHints hints,
- AF_Dimension dim );
+ af_latin_hints_compute_segments( AF_GlyphHints hints,
+ AF_Dimension dim );
+ /* this shouldn't normally be exported. However, other scripts might
+ * want to use this function as-is
+ */
FT_LOCAL( void )
- af_latin_hints_compute_edges( AF_OutlineHints hints,
- AF_Dimension dim );
+ af_latin_hints_link_segments( AF_GlyphHints hints,
+ AF_Dimension dim );
+ /* this shouldn't normally be exported. However, other scripts might
+ * want to use this function as-is
+ */
FT_LOCAL( void )
- af_latin_hints_init( AF_OutlineHints hints,
- AF_Dimension dim );
+ af_latin_hints_compute_edges( AF_GlyphHints hints,
+ AF_Dimension dim );
+ FT_LOCAL( void )
+ af_latin_hints_detect_features( AF_GlyphHints hints,
+ AF_Dimension dim );
/* */
diff --git a/src/autofit/aftypes.h b/src/autofit/aftypes.h
index a8a2b8d..0bc9eb1 100644
--- a/src/autofit/aftypes.h
+++ b/src/autofit/aftypes.h
@@ -1,6 +1,9 @@
#ifndef __AFTYPES_H__
#define __AFTYPES_H__
+#include <ft2build.h>
+#include FT_FREETYPE_H
+
FT_BEGIN_HEADER
/**************************************************************************/
@@ -27,6 +30,27 @@ FT_BEGIN_HEADER
/**************************************************************************/
/**************************************************************************/
/***** *****/
+ /***** U T I L I T Y *****/
+ /***** *****/
+ /**************************************************************************/
+ /**************************************************************************/
+
+ typedef struct AF_WidthRec_
+ {
+ FT_Pos org; /* original position/width in font units */
+ FT_Pos cur; /* current/scaled position/width in device sub-pixels */
+ FT_Pos fit; /* current/fitted position/width in device sub-pixels */
+
+ } AF_WidthRec, *AF_Width;
+
+
+ AF_LOCAL( void )
+ af_sort_pos( FT_UInt count,
+ FT_Pos* table );
+
+ /**************************************************************************/
+ /**************************************************************************/
+ /***** *****/
/***** A N G L E T Y P E S *****/
/***** *****/
/**************************************************************************/
@@ -73,13 +97,17 @@ FT_BEGIN_HEADER
/**************************************************************************/
/**************************************************************************/
- typedef struct AF_OutlineHintsRec_* AF_OutlineHints;
-
- typedef struct AF_GlobalHintsRec_* AF_GlobalHints;
+ /* opaque handle to glyph-specific hints. see "afhints.h" for more
+ * details
+ */
+ typedef struct AF_GlyphHintsRec_* AF_GlyphHints;
+ /* this structure is used to model an input glyph outline to
+ * the auto-hinter. The latter will set the "hints" field
+ * depending on the glyph's script
+ */
typedef struct AF_OutlineRec_
{
- FT_Memory memory;
FT_Face face;
FT_OutlineRec outline;
FT_UInt outline_resolution;
@@ -87,53 +115,10 @@ FT_BEGIN_HEADER
FT_Int advance;
FT_UInt metrics_resolution;
- AF_OutlineHints hints;
+ AF_GlyphHints hints;
} AF_OutlineRec;
- /**************************************************************************/
- /**************************************************************************/
- /***** *****/
- /***** G L O B A L M E T R I C S *****/
- /***** *****/
- /**************************************************************************/
- /**************************************************************************/
-
- /*
- * the following define global metrics in a _single_ dimension
- *
- * the "blue_refs" and "blue_shoots" arrays are ignored in
- * the horizontal dimension
- */
-
- typedef struct AF_WidthRec_
- {
- FT_Pos org; /* original position/width in font units */
- FT_Pos cur; /* current/scaled position/width in device sub-pixels */
- FT_Pos fit; /* current/fitted position/width in device sub-pixels */
-
- } AF_WidthRec, *AF_Width;
-
-
-#define AF_MAX_WIDTHS 16
-#define AF_MAX_BLUES 32
-
- typedef struct AF_GlobalMetricsRec_
- {
- FT_Int num_widths;
- AF_WidthRec widths[ AF_MAX_WIDTHS ];
-
- FT_Fixed scale; /* used to scale from org to cur with: */
- FT_Pos delta; /* x_cur = x_org * scale + delta */
-
- /* ignored for horizontal metrics */
- AF_WidthRec blue_refs [ AF_MAX_BLUES ];
- AF_WidthRec blue_shoots[ AF_MAX_BLUES ];
-
- FT_Bool control_overshoot;
-
- } AF_GlobalMetricsRec, *AF_GlobalMetrics;
-
/**************************************************************************/
/**************************************************************************/
@@ -190,30 +175,32 @@ FT_BEGIN_HEADER
* - a specific global analyzer that will compute global metrics
* specific to the script.
*
- * - a specific hinting routine
+ * - a specific glyph analyzer that will compute segments and
+ * edges for each glyph covered by the script
+ *
+ * - a specific grid-fitting algorithm that will distort the
+ * scaled glyph outline according to the results of the glyph
+ * analyzer
*
- * all scripts should share the same analysis routine though
+ * note that a given analyzer and/or grid-fitting algorithm can be
+ * used by more than one script
*/
typedef enum
{
AF_SCRIPT_LATIN = 0,
- /* add new scripts here */
+ /* add new scripts here. don't forget to update the list in "afglobal.c" */
AF_SCRIPT_MAX /* do not remove */
} AF_Script;
+
typedef struct AF_ScriptClassRec_ const* AF_ScriptClass;
- /*
- * root class for script-specific metrics
- */
typedef struct AF_ScriptMetricsRec_
{
- AF_ScriptClass script_class;
- AF_GlobalMetricsRec horz_metrics;
- AF_GlobalMetricsRec vert_metrics;
+ AF_ScriptClass clazz;
} AF_ScriptMetricsRec, *AF_ScriptMetrics;
@@ -221,21 +208,23 @@ FT_BEGIN_HEADER
/* this function parses a FT_Face to compute global metrics for
* a specific script
*/
- typedef FT_Error (*AF_Script_InitMetricsFunc)( AF_ScriptMetrics metrics,
- FT_Face face );
+ typedef FT_Error (*AF_Script_InitMetricsFunc)( AF_ScriptMetrics metrics,
+ FT_Face face );
typedef void (*AF_Script_ScaleMetricsFunc)( AF_ScriptMetrics metrics,
AF_Scaler scaler );
- typedef void (*AF_Script_DoneMetricsFunc)( AF_ScriptMetrics metrics );
+ typedef void (*AF_Script_DoneMetricsFunc)( AF_ScriptMetrics metrics );
- typedef FT_Error (*AF_Script_InitHintsFunc)( AF_OutlineHints hints,
+ typedef FT_Error (*AF_Script_InitHintsFunc)( AF_GlyphHints hints,
AF_Scaler scaler,
AF_ScriptMetrics metrics );
- typedef void (*AF_Script_ApplyHintsFunc)( AF_OutlineHints hints );
-
+ typedef void (*AF_Script_ApplyHintsFunc)( AF_GlyphHints hints,
+ AF_Scaler scaler,
+ AF_ScriptMetrics metrics );
+
typedef struct AF_Script_UniRangeRec_
{
@@ -248,40 +237,19 @@ FT_BEGIN_HEADER
typedef struct AF_ScriptClassRec_
{
AF_Script script;
- AF_Scipt_UniRange script_uni_ranges; /* last must be { 0, 0 } */
+ AF_Script_UniRange script_uni_ranges; /* last must be { 0, 0 } */
FT_UInt script_metrics_size;
AF_Script_InitMetricsFunc script_metrics_init;
AF_Script_ScaleMetricsFunc script_metrics_scale;
AF_Script_DoneMetricsFunc script_metrics_done;
- } AF_ScriptClassRec;
-
+ AF_Script_InitHintsFunc script_hints_init;
+ AF_Script_ApplyHintsFunc script_hints_apply;
+ } AF_ScriptClassRec;
- /**************************************************************************/
- /**************************************************************************/
- /***** *****/
- /***** F A C E G L O B A L S *****/
- /***** *****/
- /**************************************************************************/
- /**************************************************************************/
-
- /*
- * models the global hints data for a given face, decomposed into
- * script-specific items..
- *
- */
- typedef struct AF_FaceGlobalsRec_
- {
- FT_Face face;
- FT_UInt glyph_count; /* same as face->num_glyphs */
- FT_Byte* glyph_scripts; /* maps each gindex to a script */
-
- FT_ScriptMetrics metrics[ AF_SCRIPT_MAX ];
- } AF_FaceGlobalsRec, *AF_FaceGlobals;
-
/* */
FT_END_HEADER
diff --git a/src/autofit/afhints.h b/src/autofit/afhints.h
index 4d73651..2f5f4ec 100644
--- a/src/autofit/afhints.h
+++ b/src/autofit/afhints.h
@@ -6,8 +6,8 @@
FT_BEGIN_HEADER
/*
* The definition of outline hints. These are shared by all
* script analysis routines
*
*/
@@ -17,7 +17,7 @@ FT_BEGIN_HEADER
AF_DIMENSION_VERT = 1, /* y coordinates, i.e. horizontal segments & edges */
AF_DIMENSION_MAX /* do not remove */
-
+
} AF_Dimension;
@@ -30,7 +30,7 @@ FT_BEGIN_HEADER
AF_DIR_LEFT = -1,
AF_DIR_UP = 2,
AF_DIR_DOWN = -2
-
+
} AF_Direction;
@@ -38,32 +38,32 @@ FT_BEGIN_HEADER
typedef enum
{
AF_FLAG_NONE = 0,
-
+
/* point type flags */
AF_FLAG_CONIC = (1 << 0),
AF_FLAG_CUBIC = (1 << 1),
AF_FLAG_CONTROL = AF_FLAG_CONIC | AF_FLAG_CUBIC,
-
+
/* point extremum flags */
AF_FLAG_EXTREMA_X = (1 << 2),
AF_FLAG_EXTREMA_Y = (1 << 3),
-
+
/* point roundness flags */
AF_FLAG_ROUND_X = (1 << 4),
AF_FLAG_ROUND_Y = (1 << 5),
-
+
/* point touch flags */
AF_FLAG_TOUCH_X = (1 << 6),
AF_FLAG_TOUCH_Y = (1 << 7),
-
+
/* candidates for weak interpolation have this flag set */
AF_FLAG_WEAK_INTERPOLATION = (1 << 8),
-
+
/* all inflection points in the outline have this flag set */
AF_FLAG_INFLECTION = (1 << 9)
-
+
} AF_Flags;
-
+
/* edge hint flags */
typedef enum
@@ -72,7 +72,7 @@ FT_BEGIN_HEADER
AF_EDGE_ROUND = (1 << 0),
AF_EDGE_SERIF = (1 << 1),
AF_EDGE_DONE = (1 << 2)
-
+
} AF_Edge_Flags;
@@ -110,10 +110,10 @@ FT_BEGIN_HEADER
AF_Edge edge; /* the segment's parent edge */
AF_Segment edge_next; /* link to next segment in parent edge */
- AF_Segment link; /* link segment */
+ AF_Segment link; /* (stem) link segment */
AF_Segment serif; /* primary segment for serifs */
FT_Pos num_linked; /* number of linked segments */
- FT_Pos score;
+ FT_Pos score; /* used during stem matching */
AF_Point first; /* first point in edge segment */
AF_Point last; /* last point in edge segment */
@@ -131,7 +131,7 @@ FT_BEGIN_HEADER
AF_Edge_Flags flags; /* edge flags */
AF_Direction dir; /* edge direction */
FT_Fixed scale; /* used to speed up interpolation between edges */
- FT_Pos* blue_edge; /* non-NULL if this is a blue edge */
+ AF_Width blue_edge; /* non-NULL if this is a blue edge */
AF_Edge link;
AF_Edge serif;
@@ -142,7 +142,6 @@ FT_BEGIN_HEADER
AF_Segment first;
AF_Segment last;
-
} AF_EdgeRec;
@@ -150,7 +149,7 @@ FT_BEGIN_HEADER
{
FT_Int num_segments;
AF_Segment segments;
-
+
FT_Int num_edges;
AF_Edge edges;
@@ -158,8 +157,8 @@ FT_BEGIN_HEADER
} AF_AxisHintsRec, *AF_AxisHints;
-
- typedef struct AF_OutlineHintsRec_
+
+ typedef struct AF_GlyphHintsRec_
{
FT_Memory memory;
@@ -177,7 +176,7 @@ FT_BEGIN_HEADER
AF_AxisHintsRec axis[ AF_DIMENSION_MAX ];
- } AF_OutlineHintsRec;
+ } AF_GlyphHintsRec;
@@ -188,43 +187,36 @@ FT_BEGIN_HEADER
FT_LOCAL( void )
- af_outline_hints_init( AF_OutlineHints hints );
-
-
- /* used to set the (u,v) fields of each AF_Point in a AF_OutlineHints
- * object.
- */
- typedef enum AH_UV_
- {
- AH_UV_FXY, /* (u,v) = (fx,fy) */
- AH_UV_FYX, /* (u,v) = (fy,fx) */
- AH_UV_OXY, /* (u,v) = (ox,oy) */
- AH_UV_OYX, /* (u,v) = (oy,ox) */
- AH_UV_OX, /* (u,v) = (ox,x) */
- AH_UV_OY, /* (u,v) = (oy,y) */
- AH_UV_YX, /* (u,v) = (y,x) */
- AH_UV_XY /* (u,v) = (x,y) * should always be last! */
+ af_glyph_hints_init( AF_GlyphHints hints,
+ FT_Memory memory );
- } AH_UV;
- FT_LOCAL_DEF( void )
- af_outline_hints_setup_uv( AF_OutlineHints hints,
- AF_UV source );
-
- /* recomputes all AF_Point in a AF_OutlineHints from the definitions
+ /* recomputes all AF_Point in a AF_GlyphHints from the definitions
* in a source outline
*/
FT_LOCAL( FT_Error )
- af_outline_hints_reset( AF_OutlineHints hints,
- FT_Outline* outline,
- FT_Fixed x_scale,
- FT_Fixed y_scale );
+ af_glyph_hints_reset( AF_GlyphHints hints,
+ FT_Outline* outline,
+ FT_Fixed x_scale,
+ FT_Fixed y_scale,
+ FT_Pos x_delta,
+ FT_Pos y_delta );
FT_LOCAL( void )
- af_outline_hints_done( AF_OutlineHints hints );
+ af_glyph_hints_align_edge_points( AF_GlyphHints hints,
+ AF_Dimension dim );
+ FT_LOCAL( void )
+ af_glyph_hints_align_strong_points( AF_GlyphHints hints,
+ AF_Dimension dim );
+ FT_LOCAL( void )
+ af_glyph_hints_align_weak_points( AF_GlyphHints hints,
+ AF_Dimension dim );
+
+ FT_LOCAL( void )
+ af_glyph_hints_done( AF_GlyphHints hints );
/* */
@@ -6,8 +6,8 @@
FT_BEGIN_HEADER
/*
- * The definition of outline hints. These are shared by all
- * script analysis routines
+ * The definition of outline glyph hints. These are shared by all
+ * script analysis routines (until now)
*
*/